' Copyright 2008 - 2009 Jozef Vince <jozef.vince@freemap.sk>
' Copyright 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.Xml
Imports System.IO

Public Class ButtonEditorMain
    Private Shared BUTTONS_COUNT As Integer = 12
    Private Sub btnBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBrowse.Click
        With OpenFileDialog1
            .InitialDirectory = GetAppPath()
            .Filter = "buttons xml (*.xml)|*.xml"
            .FilterIndex = 2
            .RestoreDirectory = True
            .Multiselect = False
        End With

        If OpenFileDialog1.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            txtButtonPath.Text = OpenFileDialog1.FileName
            If File.Exists(txtButtonPath.Text) Then
                'tu pojde load
                Try
                    ButtonXML.LoadButtons(IO.Path.GetDirectoryName(txtButtonPath.Text), IO.Path.GetFileName(txtButtonPath.Text))
                    LoadPages()
                Catch ex As Exception
                    MsgBox("Could not load file.", MsgBoxStyle.Exclamation)
                End Try
            Else
                MsgBox("File does not exists!!!", MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly)
            End If
        End If
    End Sub
    Private Sub LoadPages()
        lstPages.Items.Clear()
        Dim i As Integer
        For i = 1 To ButtonXML.Pages.Count
            Dim b As Page
            b = ButtonXML.Pages.Item(i)
            lstPages.Items.Add(b.ID)
        Next
        lstPages.SelectedIndex = -1
        lstPages_SelectedIndexChanged(Nothing, Nothing)
        UpdateHotKeysAndHwButtons()
    End Sub
    Private Sub lstPages_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstPages.SelectedIndexChanged
        Dim pageID As Integer = 0
        cmbMissingButtons.Items.Clear()
        If lstPages.SelectedIndex > -1 Then
            pageID = CInt(lstPages.Items(lstPages.SelectedIndex))
            Dim page As Page = ButtonXML.Pages.Item(pageID)
            txtTitle.Text = page.Title
            txtGotoPage.Text = page.GotoPage.ToString()
            For i As Integer = 1 To BUTTONS_COUNT
                If Not page.Buttons.ContainsKey(i) Then
                    cmbMissingButtons.Items.Add(i)
                End If
            Next
        End If
        ButtonXML.CurrentPageID = pageID
        ButtonPreview.ButtonPanel1.SetupPage(ButtonPreview.CheckBox1.Checked)
        pageFrame.Visible = lstPages.SelectedIndex > -1
        toggleButtons()
    End Sub
    Private Sub btnDelPage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelPage.Click
        If lstPages.SelectedIndex > -1 Then
            ' Check that there are no references to the page to delete left
            Dim i, j As Integer
            Dim pageIDToDelete As Integer = CInt(lstPages.Items(lstPages.SelectedIndex))
            For i = 1 To ButtonXML.Pages.Count
                Dim b As Page
                b = ButtonXML.Pages.Item(i)
                For j = 0 To b.Buttons.Count - 1
                    ' TODO: Prev/Next Buttons?
                    If b.Buttons.ContainsKey(j) AndAlso b.Buttons.Item(j).GotoPage = pageIDToDelete Then
                        MsgBox("The page you want to delete is reverenced on page " & b.ID & " in button " & (j + 1), MsgBoxStyle.Exclamation)
                        Exit Sub
                    End If
                Next
            Next
            ' Do the removal
            Dim newPages As Pages = New Pages
            For i = 1 To pageIDToDelete - 1
                newPages.Add(i, ButtonXML.Pages.Item(i))
            Next
            For i = pageIDToDelete + 1 To ButtonXML.Pages.Count
                Dim b As Page
                b = ButtonXML.Pages.Item(i)
                b.ID -= 1
                newPages.Add(b.ID, b)
                For j = 1 To b.Buttons.Count
                    If b.Buttons.ContainsKey(j) AndAlso b.Buttons.Item(j).GotoPage > pageIDToDelete Then
                        b.Buttons.Item(j).GotoPage -= 1
                    End If
                Next
            Next
            ButtonXML.Pages = newPages
            LoadPages()
        End If
    End Sub
    Private Sub btnAddPage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAddPage.Click
        Dim i As Integer = ButtonXML.Pages.Count + 1
        Dim page As New Page()
        page.ID = ButtonXML.Pages.Count + 1
        page.Title = InputBox("Page Title", "New Page")
        page.PageMode = PagePresentationMode.DefaultAutoSize
        ButtonXML.Pages.Add(page.ID, page)
        LoadPages()
    End Sub
    Private Sub toggleButtons()
        btnDelPage.Enabled = lstPages.SelectedIndex > -1
        btnUp.Enabled = lstPages.SelectedIndex >= 1
        btnDown.Enabled = lstPages.SelectedIndex > -1 AndAlso lstPages.SelectedIndex < lstPages.Items.Count - 1
    End Sub
    Private Sub btnUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnUp.Click
        If lstPages.SelectedIndex >= 1 Then
            SwapPages(lstPages.SelectedIndex + 1)
        End If
    End Sub
    Private Sub btnDown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDown.Click
        If lstPages.SelectedIndex > -1 AndAlso lstPages.SelectedIndex < lstPages.Items.Count - 1 Then
            SwapPages(lstPages.SelectedIndex + 1 + 1)
        End If
    End Sub
    Private Sub SwapPages(ByVal pageID As Integer)
        ' TODO: Prev/Next Buttons -> goto?
        Dim i, j As Integer
        Dim newPages As Pages = New Pages

        Dim pageToMoveUp As Page = ButtonXML.Pages.Item(pageID)
        Dim pageToMoveDown As Page = ButtonXML.Pages.Item(pageID - 1)

        ' Update all pages before
        For i = 1 To pageID - 2
            newPages.Add(i, ButtonXML.Pages.Item(i))
            Dim page As Page = ButtonXML.Pages.Item(i)
            For j = 1 To page.Buttons.Count
                If page.Buttons.ContainsKey(j) AndAlso page.Buttons.Item(j).GotoPage = pageToMoveUp.ID Then
                    page.Buttons.Item(j).GotoPage = pageToMoveDown.ID
                ElseIf page.Buttons.ContainsKey(j) AndAlso page.Buttons.Item(j).GotoPage = pageToMoveDown.ID Then
                    page.Buttons.Item(j).GotoPage = pageToMoveUp.ID
                End If
            Next
        Next

        ' Do the swap
        Dim oldID As Integer = pageToMoveUp.ID
        pageToMoveUp.ID = pageToMoveDown.ID
        pageToMoveDown.ID = oldID
        newPages.Add(pageToMoveUp.ID, pageToMoveUp)
        newPages.Add(pageToMoveDown.ID, pageToMoveDown)
        For j = 1 To pageToMoveUp.Buttons.Count
            If pageToMoveUp.Buttons.ContainsKey(j) AndAlso pageToMoveUp.Buttons.Item(j).GotoPage = pageToMoveUp.ID Then
                pageToMoveUp.Buttons.Item(j).GotoPage = pageToMoveDown.ID
            ElseIf pageToMoveUp.Buttons.ContainsKey(j) AndAlso pageToMoveUp.Buttons.Item(j).GotoPage = pageToMoveDown.ID Then
                pageToMoveUp.Buttons.Item(j).GotoPage = pageToMoveUp.ID
            End If
        Next
        For j = 1 To pageToMoveDown.Buttons.Count
            If pageToMoveDown.Buttons.ContainsKey(j) AndAlso pageToMoveDown.Buttons.Item(j).GotoPage = pageToMoveUp.ID Then
                pageToMoveDown.Buttons.Item(j).GotoPage = pageToMoveDown.ID
            ElseIf pageToMoveDown.Buttons.ContainsKey(j) AndAlso pageToMoveDown.Buttons.Item(j).GotoPage = pageToMoveDown.ID Then
                pageToMoveDown.Buttons.Item(j).GotoPage = pageToMoveUp.ID
            End If
        Next

        ' update all following pages
        For i = pageID + 1 To ButtonXML.Pages.Count
            Dim page As Page = ButtonXML.Pages.Item(i)
            newPages.Add(page.ID, page)
            For j = 1 To page.Buttons.Count
                If page.Buttons.ContainsKey(j) AndAlso page.Buttons.Item(j).GotoPage = pageToMoveUp.ID Then
                    page.Buttons.Item(j).GotoPage = pageToMoveDown.ID
                ElseIf page.Buttons.ContainsKey(j) AndAlso page.Buttons.Item(j).GotoPage = pageToMoveDown.ID Then
                    page.Buttons.Item(j).GotoPage = pageToMoveUp.ID
                End If
            Next
        Next

        ButtonXML.Pages = newPages
        LoadPages()
    End Sub
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSavePageTitle.Click
        If ButtonXML.Pages.ContainsKey(ButtonXML.CurrentPageID) Then
            Dim page As Page = ButtonXML.Pages.Item(ButtonXML.CurrentPageID)
            page.Title = txtTitle.Text
            page.GotoPage = Integer.Parse(txtGotoPage.Text)
            ButtonPreview.ButtonPanel1.SetupPage(ButtonPreview.CheckBox1.Checked)
        End If
    End Sub
    Private Sub btnAddBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAddBtn.Click
        If cmbMissingButtons.SelectedIndex > -1 Then
            btnAddBtn.Enabled = False
            Dim page As Page = ButtonXML.Pages.Item(ButtonXML.CurrentPageID)
            Dim button As New Button
            button.ID = CInt(cmbMissingButtons.Items.Item(cmbMissingButtons.SelectedIndex))
            button.Tag = ""
            button.Title = ""
            page.Buttons.Add(button.ID, button)
            ButtonPreview.ButtonPanel1.SetupPage(ButtonPreview.CheckBox1.Checked)
            cmbMissingButtons.Items.RemoveAt(cmbMissingButtons.SelectedIndex)
            cmbMissingButtons.SelectedIndex = -1
        End If
    End Sub
    Private Sub cmbMissingButtons_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbMissingButtons.SelectedIndexChanged
        btnAddBtn.Enabled = cmbMissingButtons.SelectedIndex > -1
    End Sub
    Private Sub btnSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSave.Click
        If txtButtonPath.Text <> "" AndAlso MsgBox("Save as " & txtButtonPath.Text & "?", MsgBoxStyle.Question Or MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            ButtonXML.SaveButtons(IO.Path.GetDirectoryName(txtButtonPath.Text), IO.Path.GetFileName(txtButtonPath.Text))
        End If
    End Sub
    Private Sub Button1_Click_1(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        frmAbout.ShowDialog()
    End Sub
    Private Sub ButtonEditorMain_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Show()
        ButtonPreview.Show()
    End Sub
    Public Sub UpdateHotKeysAndHwButtons()
        hotkeys.Items.Clear()
        For Each keyaction As KeyAction In ButtonXML.KeyActions
            If keyaction.HotKey <> Nothing Then
                hotkeys.Items.Add(keyaction.HotKey & " " & keyaction.btn.ToString)
            End If
        Next
        hwbuttons.Items.Clear()
        For i As Integer = 0 To ButtonXML.HWButtonActions.GetLength(0) - 1
            If ButtonXML.HWButtonActions(i) IsNot Nothing Then
                hwbuttons.Items.Add((i + 1) & " " & ButtonXML.HWButtonActions(i).ToString)
            End If
        Next
    End Sub
End Class
