' Copyright 2008 Jozef Vince <jozef.vince@freemap.sk>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.


Option Explicit On
Option Strict On

Imports System.Drawing.Imaging

Public Class ButtonPanel

    Public Event ButtonHit(ByVal tag As String, ByVal title As String, ByVal gotopage As Integer)
    Public Event ButtonHitEx(ByRef button As Button)

    ' Fields
    Private borderPen As Pen = New Pen(Color.Black)
    Private CurrentButton As Button
    Private CurrentPage As Page

    Private DimBrush As SolidBrush = New SolidBrush(Color.FromArgb(&H8B, &H9F, &HB2))
    Private DimPen As Pen = New Pen(Color.FromArgb(&H8B, &H9F, &HB2))
    Private WhiteBrush As SolidBrush = New SolidBrush(Color.White)
    Private WhitePen As Pen = New Pen(Color.White)

    Private BorderHighlightPen As New Pen(Color.LightGray, 1)
    Private BorderShadowPen As New Pen(Color.FromArgb(&H4E, &H4E, &H4E))
    Private DisabledBorderHighlightPen As Pen = DimPen
    Private DisabledBorderShadowPen As New Pen(Color.FromArgb(&H70, &H70, &H70))

    Private txtFont As Font = New Font("Arial", 9, FontStyle.Bold)
    Private txtSmallFont As Font = New Font("Arial", 7, FontStyle.Regular)

    Private PagePrerendered As Boolean = False

    Private m_bmpOffscreen As Bitmap
    Private _PageNameBarHeight As Integer = 0

#If SmartPhone Then
    Private Shared btnMap As String() = {"", "1", "2", "3", "4", "5", "6", "7", "8", "9", "*", "0", "#"}
#End If

    Private Function BackgroundImageColor(ByVal image As Image) As Color
        Dim bitmap As New Bitmap(image)
        BackgroundImageColor = bitmap.GetPixel(0, 0)
        bitmap.Dispose()
    End Function

    Private Sub ButtonPanel_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        Dim x As Integer = e.X
        Dim y As Integer = e.Y
        If (Not CurrentPage Is Nothing) Then
            Dim button As Button
            For Each button In CurrentPage.Buttons.Values
                If button.Enabled Then
                    If (x > button.Geometry.Left) AndAlso (x < button.Geometry.Right) AndAlso _
                       (y > button.Geometry.Top) AndAlso (y < button.Geometry.Bottom) Then
                        Dim gxOff As Graphics = Graphics.FromImage(m_bmpOffscreen)

                        If (CurrentPage.PageMode <> PagePresentationMode.BackgroundImage) Then
                            RenderButton(gxOff, button, True)
                        Else
                            Dim clientRectangle As Rectangle = clientRectangle
                            clientRectangle.Width -= 1
                            clientRectangle.Height -= 1
                            gxOff.DrawRectangle(WhitePen, clientRectangle)
                        End If
                        gxOff.Dispose()
                        CurrentButton = button
                        Invalidate()
                    End If
                End If
            Next
        End If
    End Sub

    Private Sub ButtonPanel_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        If (Not CurrentButton Is Nothing) Then
            Dim x As Integer = e.X
            Dim y As Integer = e.Y
            If (Not CurrentPage Is Nothing) Then
                Dim gxOff As Graphics = Graphics.FromImage(m_bmpOffscreen)
                If (CurrentPage.PageMode <> PagePresentationMode.BackgroundImage) Then
                    RenderButton(gxOff, CurrentButton, False)
                    Invalidate()
                Else
                    Dim clientRectangle As Rectangle = clientRectangle
                    clientRectangle.Width -= 1
                    clientRectangle.Height -= 1
                    gxOff.DrawRectangle(DimPen, clientRectangle)
                    Invalidate()
                End If
                gxOff.Dispose()
                If (x > CurrentButton.Geometry.Left) AndAlso (x < CurrentButton.Geometry.Right) AndAlso _
                   (y > CurrentButton.Geometry.Top) AndAlso (y < CurrentButton.Geometry.Bottom) Then
                    RaiseEvent ButtonHitEx(CurrentButton)
                End If
            End If
        End If
        CurrentButton = Nothing
    End Sub

    Private Sub ButtonPanel_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        If m_bmpOffscreen IsNot Nothing Then
            m_bmpOffscreen.Dispose()
        End If
        m_bmpOffscreen = Nothing
        PagePrerendered = False
        Invalidate()
    End Sub

    Protected Overrides Sub OnPaint(ByVal pe As PaintEventArgs)
        If Not PagePrerendered Then
            PrerenderPage()
        End If
        pe.Graphics.DrawImage(m_bmpOffscreen, 0, 0)
        MyBase.OnPaint(pe)
    End Sub

    Protected Overrides Sub OnPaintBackground(ByVal pe As PaintEventArgs)
        ' prevent flickering
    End Sub

    Private Sub AbjustPageNameBarHeight(ByVal h As Integer)
        If h <> _PageNameBarHeight Then
            If h < 16 Then
                _PageNameBarHeight = 16
            Else
                _PageNameBarHeight = 32
            End If
            If (Not CurrentPage Is Nothing) Then
                If (CurrentPage.PageMode = PagePresentationMode.DefaultAutoSize) Then
                    RepositionButtons()
                End If
            End If
        End If
    End Sub

    Private Sub PrerenderPage()
        Dim h As Integer
        Dim w As Integer
        Dim button As Button
        Dim pageTitle As String = "FONT HEIGHT MEASURE STRING"

        Dim cRectangle As Rectangle = ClientRectangle
        PagePrerendered = True
        If (m_bmpOffscreen Is Nothing) Then
            m_bmpOffscreen = New Bitmap(ClientSize.Width, ClientSize.Height, Imaging.PixelFormat.Format16bppRgb565)

            Dim current As Page
            For Each current In ButtonXML.Pages.Values
                If (current.PageMode = PagePresentationMode.DefaultAutoSize) Then
                    current.IsPrepared = False
                End If
            Next
        End If

        Dim gxOff As Graphics = Graphics.FromImage(m_bmpOffscreen)
        gxOff.Clear(BackColor)
        h = gxOff.MeasureString(pageTitle, txtFont).ToSize.Height
        AbjustPageNameBarHeight(h)


        If (CurrentPage Is Nothing) Then
            pageTitle = "No Page Selected"
            w = gxOff.MeasureString(pageTitle, txtFont).ToSize.Width
            ' gxOff.FillRectangle(DimBrush, 0, 0, cRectangle.Width, _PageNameBarHeight)
            ' gxOff.DrawLine(borderPen, 0, _PageNameBarHeight, cRectangle.Width, _PageNameBarHeight)
            gxOff.DrawString(pageTitle, txtFont, WhiteBrush, CSng(((ClientSize.Width - w) >> 1)), CSng((_PageNameBarHeight - h)))
        Else
            Select Case CurrentPage.PageMode
                Case PagePresentationMode.DefaultAutoSize
                    w = gxOff.MeasureString(CurrentPage.Title, txtFont).ToSize.Width
                    gxOff.DrawString(CurrentPage.Title, txtFont, WhiteBrush, CSng(((ClientSize.Width - w) >> 1)), CSng((_PageNameBarHeight - h)))

                    Dim i As Integer
                    For i = 1 To 12
                        If CurrentPage.Buttons.ContainsKey(i) Then
                            button = CurrentPage.Buttons.Item(i)
                            RenderButton(gxOff, button, False)
                        End If
                    Next i

                Case PagePresentationMode.Fixed
                    w = gxOff.MeasureString(CurrentPage.Title, txtFont).ToSize.Width
                    gxOff.DrawString(CurrentPage.Title, txtFont, WhiteBrush, CSng(((ClientSize.Width - w) >> 1)), CSng((_PageNameBarHeight - h)))

                    For Each button In CurrentPage.Buttons.Values
                        RenderButton(gxOff, button, False)
                    Next
                Case PagePresentationMode.BackgroundImage
                    gxOff.DrawImage(CurrentPage.PageImage, 0, 0)
            End Select
            cRectangle.Width -= 1
            cRectangle.Height -= 1
            gxOff.DrawRectangle(borderPen, ClientRectangle)
        End If
        gxOff.Dispose()
    End Sub

    Private Sub RenderButton(ByRef gxOff As Graphics, ByRef Button As Button, ByVal pressed As Boolean)
        Dim pen1 As Pen
        Dim pen2 As Pen
        Dim usedFont As Font
        Dim image As New Bitmap(Button.Geometry.Width, Button.Geometry.Height)
        Dim graphics As Graphics = graphics.FromImage(image)
        If pressed Then
            graphics.Clear(Color.Red)
        Else
            graphics.Clear(BackColor)
        End If
        If ((Not Button.Icon Is Nothing) AndAlso Button.Enabled) Then
            Dim w As Integer = ((image.Width - Button.Icon.Width) >> 1)
            Dim h As Integer = ((image.Height - Button.Icon.Height) >> 1)
            Dim destRect As New Rectangle(w, h, Button.Icon.Width, Button.Icon.Height)
            Dim imageAttr As New ImageAttributes
            Dim colorLow As Color = BackgroundImageColor(Button.Icon)
            imageAttr.SetColorKey(colorLow, colorLow)
            graphics.DrawImage(Button.Icon, destRect, 0, 0, Button.Icon.Width, Button.Icon.Height, GraphicsUnit.Pixel, imageAttr)
        End If

        Dim x As Integer, y As Integer
        Dim stringsize As SizeF

#If SmartPhone Then
        graphics.DrawString(btnMap(Button.ID), txtSmallFont, WhiteBrush, 2, 1)
#End If

        If (Button.Icon Is Nothing) Then
            usedFont = txtFont
            stringsize = gxOff.MeasureString(Button.Title, usedFont)
            x = (Button.Geometry.Width - CInt(stringsize.Width)) >> 1
            y = (Button.Geometry.Height - CInt(stringsize.Height)) >> 1
        Else
            usedFont = txtSmallFont
            stringsize = gxOff.MeasureString(Button.Title, usedFont)
            x = (Button.Geometry.Width - CInt(stringsize.Width)) >> 1
            y = (Button.Geometry.Height - CInt(stringsize.Height))
        End If

        If Button.Enabled Then
            graphics.DrawString(Button.Title, usedFont, WhiteBrush, x, y)
        Else
            graphics.DrawString(Button.Title, usedFont, DimBrush, x, y)
        End If


        If Button.Enabled Then
            If pressed Then
                pen2 = BorderShadowPen
                pen1 = BorderHighlightPen
            Else
                pen1 = BorderShadowPen
                pen2 = BorderHighlightPen
            End If
        Else
            pen1 = DisabledBorderShadowPen
            pen2 = DisabledBorderHighlightPen
        End If

        Dim points As Point() = New Point(3 - 1) {}
        x = (Button.Geometry.Width - 1)
        y = (Button.Geometry.Height - 1)
        points(0) = New Point(0, y)
        points(1) = New Point(x, y)
        points(2) = New Point(x, 0)
        graphics.DrawLines(pen1, points)
        points(0) = New Point(0, y)
        points(1) = New Point(0, 0)
        points(2) = New Point(x + 1, 0)
        graphics.DrawLines(pen2, points)

        gxOff.DrawImage(image, Button.Geometry.X, Button.Geometry.Y)
        graphics.Dispose()
        image.Dispose()
    End Sub

    Private Sub RepositionButtons()
        CurrentPage.UpdateAutoSizeLayout(Width, Height, _PageNameBarHeight)
    End Sub

    Public Sub SetupPage(ByVal gps_fixed As Boolean)
        PagePrerendered = False
        CurrentButton = Nothing
        If ButtonXML.Pages.ContainsKey(ButtonXML.CurrentPageID) Then
            CurrentPage = ButtonXML.Pages.Item(ButtonXML.CurrentPageID)
            If (Not CurrentPage Is Nothing) Then
                If ((CurrentPage.PageMode = PagePresentationMode.DefaultAutoSize) AndAlso Not CurrentPage.IsPrepared) Then
                    RepositionButtons()
                End If
                Dim button As Button
                For Each button In CurrentPage.Buttons.Values
                    button.Enabled = (gps_fixed OrElse button.Tag.StartsWith("__"))
                Next
            End If
        End If
        Invalidate()
    End Sub
End Class
