' Copyright 2007 Rubke <interrubke@gmail.com>
' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
' Copyright 2008 - 2009 Jozef Vince <jozef.vince@freemap.sk>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.Xml
Imports System.IO

Public Class ButtonXML
    ''' <summary>
    ''' Collection of available pages
    ''' </summary>
    Public Shared Pages As Pages = New Pages
    ''' <summary>
    ''' Selected page id
    ''' </summary>
    Public Shared CurrentPageID As Integer = 1
    ''' <summary>
    ''' Number of HW buttons (6 by default)
    ''' </summary>
    Public Const HardwareButtons As Integer = 6
    ''' <summary>
    ''' Array of HW button actions
    ''' </summary>
    Public Shared HWButtonActions(HardwareButtons - 1) As Button
    ''' <summary>
    ''' Linked list of key actions
    ''' </summary>
    Public Shared KeyActions As LinkedList(Of KeyAction) = New LinkedList(Of KeyAction)


    ''' <summary>
    ''' Loads the button definition file and build in-memory data structures.
    ''' </summary>
    ''' <param name="storageFolder">Path where data is stored.</param>
    ''' <param name="buttonFileName">Name of the button file.</param>
    Public Shared Sub LoadButtons(ByVal storageFolder As String, Optional ByVal buttonFileName As String = "buttons.xml")
        Dim btn As New Button
        Dim pg As Page

        Dim oButtonsDoc As New XmlDocument
        Dim appFolder As String = GetAppPath()
        Dim fullFileName As String = storageFolder & Path.DirectorySeparatorChar & buttonFileName
        If Not IO.File.Exists(fullFileName) Then
            storageFolder = appFolder
            fullFileName = storageFolder & Path.DirectorySeparatorChar & buttonFileName
        End If
        If IO.File.Exists(fullFileName) Then
            oButtonsDoc.Load(fullFileName)

            Pages.Clear()
            KeyActions.Clear()
            Dim i As Integer
            For i = 0 To HardwareButtons - 1
                HWButtonActions(i) = Nothing
            Next

            Pages.Load(oButtonsDoc.SelectSingleNode("pages"), storageFolder)

            ' extract performance helpers 
            For Each pg In Pages.Values
                For Each btn In pg.Buttons.Values
                    If btn.HotKey <> "" Then
                        AddHotKeyMapping(btn)
                    End If
                    If btn.HardKey > 0 AndAlso btn.HardKey <= HardwareButtons Then
                        AddHwBtnMapping(btn)
                    End If
                Next
            Next
        End If
    End Sub
    Public Shared Sub AddHotKeyMapping(ByVal btn As Button)
        If Not KeyActions.Contains(New KeyAction(btn.HotKey)) Then
            KeyActions.AddLast(New KeyAction(btn.HotKey, btn))
        Else
            KeyActions.Find(New KeyAction(btn.HotKey)).Value.btn = btn
        End If
    End Sub
    Public Shared Sub RemoveHotKeyMapping(ByVal btn As Button)
        KeyActions.Remove(New KeyAction(btn.HotKey))
    End Sub
    Public Shared Sub AddHwBtnMapping(ByVal btn As Button)
        HWButtonActions(btn.HardKey - 1) = btn
    End Sub
    Public Shared Sub RemoveHwBtnMapping(ByVal btnNumber As Integer)
        HWButtonActions(btnNumber - 1) = Nothing
    End Sub
#If ButtonEditor Then
    ''' <summary>
    ''' Saves the buttons.
    ''' </summary>
    ''' <param name="storageFolder">The storage folder.</param>
    ''' <param name="buttonFileName">Name of the button file.</param>
    Public Shared Sub SaveButtons(ByVal storageFolder As String, Optional ByVal buttonFileName As String = "buttons.xml")
        Dim xmlDoc As New XmlDocument

        Dim oPages As XmlElement = xmlDoc.CreateElement("pages")
        xmlDoc.AppendChild(oPages)

        ButtonXML.Pages.Save(xmlDoc, oPages)

        xmlDoc.Save(storageFolder & Path.DirectorySeparatorChar & buttonFileName)
    End Sub
#End If

    Private Shared FixIndependentCommandsList() As String = {"about", "next", "prev", "downloadtiles", "settings", "exit", _
                  "zoomin", "zoomout", "tracking", "drawtrack", "about", "map", "gps", _
                  "data", "waypoints", "goto"}

    ' always adjust fix-aware-command-names as "__" - so they always show up 
    Public Shared Function NormalizeCommand(ByVal text As String) As String
        Dim name As String, facn As String
        If text.StartsWith("_") Then
            name = text.Replace("_", "")
            For Each facn In FixIndependentCommandsList
                If name = facn Then
                    Return "__" & name
                End If
            Next
        End If
        Return text
    End Function
End Class
