' Copyright 2007 Rubke <interrubke@gmail.com>
' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
' Copyright 2008 - 2009 Jozef Vince <jozef.vince@freemap.sk>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO
Imports System.Xml

Public Class Button
    Public ID As Integer
    Public Title As String
    Public Tag As String
    Public GotoPage As Integer
    Public HotKey As Char
    Public HardKey As Integer
    Public Icon As Bitmap
    Public ExecPath As String

    Public Enabled As Boolean = False
    Public SoundFile As String

#If ButtonEditor Then
    ' raw data from buttons.xml, SoundFile get's path prefixed on loading
    Public IconFileName As String
    Public SoundFileName As String
#End If

    Public Geometry As System.Drawing.Rectangle = New System.Drawing.Rectangle(0, 0, 32, 32)

    Public Overrides Function Equals(ByVal otherObject As Object) As Boolean
        If Not otherObject.GetType.Equals(GetType(Button)) Then
            Equals = False
        Else
            If DirectCast(otherObject, Button).ID = ID Then
                Equals = True
            Else
                Equals = False
            End If
        End If
    End Function

    ''' <summary>
    ''' Loads the specified button.
    ''' </summary>
    ''' <param name="oButton">The button.</param>
    ''' <param name="storageFolder">The storage folder.</param>
    Public Sub Load(ByRef oButton As XmlNode, ByVal storageFolder As String)
        Dim appFolder As String = GetAppPath()

        Tag = ButtonXML.NormalizeCommand(oButton.InnerText)
        If Not oButton.Attributes("id") Is Nothing Then ID = Integer.Parse(oButton.Attributes("id").InnerText)
        If Not oButton.Attributes("title") Is Nothing Then Title = oButton.Attributes("title").InnerText.Replace("\n", vbCrLf)
        If Not oButton.Attributes("gotoPage") Is Nothing Then GotoPage = Integer.Parse(oButton.Attributes("gotoPage").InnerText)
        If Not oButton.Attributes("execPath") Is Nothing Then ExecPath = oButton.Attributes("execPath").InnerText
        If Not oButton.Attributes("hotkey") Is Nothing Then
            HotKey = oButton.Attributes("hotkey").InnerText.Chars(0)
        End If
        If Not oButton.Attributes("sound") Is Nothing Then
            SoundFile = storageFolder & IO.Path.DirectorySeparatorChar & "snd" & IO.Path.DirectorySeparatorChar & oButton.Attributes("sound").InnerText
#If ButtonEditor Then
            SoundFileName = SoundFile
#End If
            If File.Exists(SoundFile) = False Then
                SoundFile = appFolder & IO.Path.DirectorySeparatorChar & "snd" & IO.Path.DirectorySeparatorChar & oButton.Attributes("sound").InnerText
                If File.Exists(SoundFile) = False Then
                    SoundFile = ""
                End If
            End If
        End If

        If Not oButton.Attributes("hardkey") Is Nothing Then
            HardKey = Integer.Parse(oButton.Attributes("hardkey").InnerText)
            ' store into extra datastructure for better performance
        End If
        If Not oButton.Attributes("icon") Is Nothing Then
            Dim strButtonIcon As String = oButton.Attributes("icon").InnerText
            Dim strIconFileName As String = storageFolder & IO.Path.DirectorySeparatorChar & "img" & IO.Path.DirectorySeparatorChar & strButtonIcon
#If ButtonEditor Then
            IconFileName = strButtonIcon
#End If
            If File.Exists(strIconFileName) = False Then
                strIconFileName = appFolder & IO.Path.DirectorySeparatorChar & "img" & IO.Path.DirectorySeparatorChar & strButtonIcon
            End If
            If File.Exists(strIconFileName) Then
                Icon = New System.Drawing.Bitmap(strIconFileName)
            End If
        End If
        If Not oButton.Attributes("x") Is Nothing AndAlso _
           Not oButton.Attributes("y") Is Nothing AndAlso _
           Not oButton.Attributes("w") Is Nothing AndAlso _
           Not oButton.Attributes("h") Is Nothing Then
            Geometry = New Rectangle(Integer.Parse(oButton.Attributes("x").InnerText), Integer.Parse(oButton.Attributes("y").InnerText), Integer.Parse(oButton.Attributes("w").InnerText), Integer.Parse(oButton.Attributes("h").InnerText))
        Else
            ' if some button doesn't have size and location defined, automatically fallback to non-extended layout
            'PageMode = PagePresentationMode.DefaultAutoSize
            ' TODO: handle situations where button def is inconsistent with page type
        End If
    End Sub

#If ButtonEditor Then
    ''' <summary>
    ''' Saves the specified button.
    ''' </summary>
    ''' <param name="oButton">The button.</param>
    Public Sub Save(ByRef xmlDoc As XmlDocument, ByRef oButton As XmlElement)

        oButton.SetAttribute("id", ID.ToString)
        oButton.SetAttribute("title", Title.Replace(vbCrLf, "\n"))
        If IconFileName IsNot Nothing AndAlso IconFileName.Length > 0 Then
            oButton.SetAttribute("icon", IconFileName)
        End If
        If GotoPage > 0 Then
            oButton.SetAttribute("gotoPage", GotoPage.ToString)
        End If
        If Not HotKey = Nothing Then
            oButton.SetAttribute("hotkey", HotKey.ToString)
        End If
        If HardKey > 0 AndAlso HardKey <= ButtonXML.HardwareButtons Then
            oButton.SetAttribute("hardkey", HardKey.ToString)
        End If
        If ExecPath IsNot Nothing AndAlso ExecPath.Length > 0 Then
            oButton.SetAttribute("execPath", ExecPath)
        End If
        If SoundFileName IsNot Nothing AndAlso SoundFileName.Length > 0 Then
            oButton.SetAttribute("sound", SoundFileName)
        End If
        oButton.InnerText = Tag
    End Sub
    Public Overrides Function ToString() As String
        Dim returnString As String = ""
        returnString = Replace(Title, vbCrLf, " ") + ", " + Tag
        If GotoPage > 0 Then
            returnString += ", Page: " & GotoPage
        End If
        If ExecPath IsNot Nothing AndAlso ExecPath <> "" Then
            returnString += ", Path: " & ExecPath
        End If
        Return returnString
    End Function
#End If
End Class

Public Class KeyAction
    Public HotKey As Char
    Public btn As Button
    Public Sub New(ByVal hotkey As Char)
        Me.HotKey = hotkey
    End Sub
    Public Sub New(ByVal hotKey As Char, ByVal btn As Button)
        Me.HotKey = hotKey
        Me.btn = btn
    End Sub
    Public Overrides Function Equals(ByVal otherObject As Object) As Boolean
        If Not otherObject.GetType.Equals(GetType(KeyAction)) Then
            Equals = False
        Else
            If DirectCast(otherObject, KeyAction).HotKey = HotKey Then
                Equals = True
            Else
                Equals = False
            End If
        End If
    End Function
End Class

''' <summary>
''' Type definition for strongly typed Buttons Collection
''' </summary>
Public Class ButtonsCollection
    Inherits System.Collections.Generic.Dictionary(Of Integer, Button)

    ''' <summary>
    ''' Loads the specified o buttons.
    ''' </summary>
    ''' <param name="oButtons">The buttons.</param>
    ''' <param name="storageFolder">The storage folder.</param>
    Public Sub Load(ByRef oButtons As XmlNode, ByVal storageFolder As String)
        Dim oButton As XmlNode
        Dim appFolder As String = GetAppPath()

        For Each oButton In oButtons.SelectNodes("button")
            Dim button As New Button
            button.Load(oButton, storageFolder)
#If ButtonEditor Then
            If Me.ContainsKey(button.ID) = False Then
                Me.Add(button.ID, button)
            End If
#Else
            ' do not show _takepicture on < wm5
            If Not (button.Tag.ToLower = "_takepicture" AndAlso Not IsWindowsMobile5OrHigher()) Then
                If Me.ContainsKey(button.ID) = False Then
                    Me.Add(button.ID, button)
                End If
            End If
#End If
        Next
    End Sub

#If ButtonEditor Then
    ''' <summary>
    ''' Saves the specified o buttons.
    ''' </summary>
    ''' <param name="oButtons">The buttons.</param>
    Public Sub Save(ByRef xmlDoc As XmlDocument, ByRef oButtons As XmlElement)
        Dim button As Button
        Dim i As Integer
        Dim oButton As XmlElement

        For Each i In Keys
            button = Item(i)
            oButton = xmlDoc.CreateElement("button")
            oButtons.AppendChild(oButton)
            button.Save(xmlDoc, oButton)
        Next
    End Sub
#End If
End Class