' Copyright 2008 Jozef Vince <jozef.vince@freemap.sk>
' Copyright 2009 Jean-Pierre Cadiat <csu333-osmtracker@yahoo.fr>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.
' 

Imports System
Imports System.Threading
Imports System.Runtime.InteropServices

''' <summary>
''' Implement timer for preventing device to suspend while tracking
''' </summary>
Public Class DeviceSleep
    ''' <summary>
    ''' The device power states
    ''' </summary>
    Public Enum PowerState
        PwrDeviceUnspecified = -1
        D0 = 0  'Full On. This is the state in which the device is on and running. It is receiving full power from the system and is delivering full functionality to the user.
        D1      'Low On. This is the state in which the device is fully functional at a lower power state than D0, a lower performance state than D0, or both. This state is applicable when the device is being used, but where peak performance is unnecessary and power is at a premium.
        D2      'Standby. This is the state in which the device is partially powered with automatic wakeup on request. The device is effectively standing by.
        D3      'Sleep. This is the state in which the device is partially powered with device-initiated wakeup if available. A device in state D3 is sleeping but capable of raising the System Power State on its own. It consumes only enough power to be able to do so; which must be less than or equal to the amount of power used in state D2
        D4      'Off. This is the state in which the device is not powered. A device in state D4 should not be consuming any significant power. Some peripheral busses require static terminations that intrinsically use non-zero power when a device is physically connected to the bus; a device on such a bus can still support D4.
        PwrDeviceMaximum
    End Enum
    ''' <summary>
    ''' The video power states
    ''' </summary>
    Public Enum VideoPowerState
        VideoPowerOn = 1
        VideoPowerStandBy
        VideoPowerSuspend
        VideoPowerOff
    End Enum
    ''' <summary>
    ''' True if another touch of SystemIdleTimerReset is required
    ''' </summary>
    Private Shared nextTouch As Boolean = False
    ''' <summary>
    ''' Timer for sleep prevention, sleep for 30 sec, touch the SystemIdleTimerReset API and sleep again
    ''' </summary>
    Private Shared noSleepTimer As System.Threading.Timer = Nothing

    <DllImport("coredll.dll", EntryPoint:="SystemIdleTimerReset", SetLastError:=True, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Sub SystemIdleTimerReset()
    End Sub
    <DllImport("coredll.dll", EntryPoint:="ExtEscape", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function ExtEscapeSet(ByVal hdc As IntPtr, ByVal nEscape As Integer, ByVal cbInput As Integer, ByRef lpszInData As Integer, ByVal cbOutput As Integer, ByVal lpszOutData As IntPtr) As Integer
    End Function
    <DllImport("coredll.dll", EntryPoint:="ExtEscape", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function ExtEscapeSet(ByVal hdc As IntPtr, ByVal nEscape As Integer, ByVal cbInput As Integer, ByRef lpszInData As VideoPowerManagement, ByVal cbOutput As Integer, ByRef lpszOutData As VideoPowerManagement) As Integer
    End Function
    <DllImport("coredll.dll", EntryPoint:="GetDC", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function GetDC(ByVal hwnd As IntPtr) As IntPtr
    End Function
    <DllImport("coredll.dll", EntryPoint:="ReleaseDC", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function ReleaseDC(ByVal hwnd As IntPtr, ByVal hDC As IntPtr) As IntPtr
    End Function
    <DllImport("coredll.dll", EntryPoint:="SetPowerRequirement", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function SetPowerRequirement(ByVal DeviceName As String, ByVal state As PowerState, ByVal dwDeviceFlags As UInteger, ByVal name As IntPtr, ByVal Reserved As ULong) As Integer
    End Function
    <DllImport("coredll.dll", EntryPoint:="ReleasePowerRequirement", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function ReleasePowerRequirement(ByVal hPowerReq As Integer) As Integer
    End Function
    <DllImport("coredll.dll", EntryPoint:="SetDevicePower", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function SetDevicePower(ByVal DeviceName As String, ByVal dwDeviceFlags As Integer, ByVal state As PowerState) As Integer
    End Function
    <DllImport("coredll.dll", EntryPoint:="GetDevicePower", SetLastError:=True, CharSet:=CharSet.Unicode, CallingConvention:=CallingConvention.Winapi)> _
    Private Shared Function GetDevicePower(ByVal DeviceName As String, ByVal dwDeviceFlags As Integer, ByRef state As PowerState) As Integer
    End Function
    ''' <summary>
    ''' Gets a value indicating whether the sleep mode prevention is ON.
    ''' </summary>
    ''' <value>
    ''' <c>true</c> if sleep mode prevention is on ; otherwise, <c>false</c>.
    ''' </value>
    Shared ReadOnly Property IsPreventionOn() As Boolean
        Get
            Return (nextTouch)
        End Get
    End Property

    ''' <summary>
    ''' Prevents entering device to sleep mode.
    ''' </summary>
    ''' <param name="prevent">
    ''' if set to <c>true</c> [prevent] = starts NoSleepTimer, otherwise stop it.
    ''' </param>
    Public Shared Sub PreventSleepMode(ByVal prevent As Boolean)

        If Not (nextTouch = prevent) Then
            If prevent = True Then  '  (prevent) - start prevention
                noSleepTimer = New Timer(New TimerCallback(AddressOf ResetIdleTimer), Nothing, 0, 30 * 1000)
            Else  ' new_mode=False (no prevent) - need to stop prevention
                If noSleepTimer IsNot Nothing Then
                    noSleepTimer.Dispose()
                    noSleepTimer = Nothing
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' TimerCallback - Resets the idle timer. 
    ''' </summary>
    ''' <param name="state">The state.</param>
    Private Shared Sub ResetIdleTimer(ByVal state As Object)
        Try
            SystemIdleTimerReset()
        Catch e As Exception
        End Try
    End Sub

    Private Structure VideoPowerManagement
        Dim Length As UInteger
        Dim DPMSVersion As UInteger
        Dim PowerState As VideoPowerState
        Public Sub Initialize()
            Length = CType(Marshal.SizeOf(Me), UInteger)
            DPMSVersion = 1
        End Sub
    End Structure

    'GDI Escapes for ExtEscape()
    Private Const QUERYESCSUPPORT As Integer = 8

    'The following are unique to CE
    Private Const GETVFRAMEPHYSICAL As Integer = 6144
    Private Const GETVFRAMELEN As Integer = 6145
    Private Const DBGDRIVERSTAT As Integer = 6146
    Private Const SETPOWERMANAGEMENT As Integer = 6147
    Private Const GETPOWERMANAGEMENT As Integer = 6148

    Private Const POWER_NAME As Integer = 1
    Private Const POWER_FORCE As Integer = 4096

    ''' <summary>
    ''' Switches the screen off. 
    ''' </summary>
    Public Shared Sub PowerOff()
        Dim hdc As IntPtr = GetDC(IntPtr.Zero)

        If ExtEscapeSet(hdc, QUERYESCSUPPORT, 4, SETPOWERMANAGEMENT, 0, IntPtr.Zero) = 0 Then
            MessageBox.Show("Sorry, your device doesn't support software screen power off.")
        Else
            Dim vpm As VideoPowerManagement
            vpm.Initialize()
            'Power state
            vpm.PowerState = VideoPowerState.VideoPowerOff
            'Set state
            Dim result As Integer = ExtEscapeSet(hdc, SETPOWERMANAGEMENT, 12, vpm, 0, Nothing)
            'Check result
            ExtEscapeSet(hdc, GETPOWERMANAGEMENT, 0, Nothing, 12, vpm)
            'On some device, if "SetDevicePower" has been called, the call to set power management
            ' gives a success (result > 0) but the screen remains on. When need to switch backlight
            ' off to correctly switch off the screen
            If result <= 0 OrElse vpm.PowerState = VideoPowerState.VideoPowerOn Then
                ToggleBackLight()
                'TODO: We will probably get user returns telling that the backlight stays off
            End If
        End If
        ReleaseDC(IntPtr.Zero, hdc)
    End Sub
    ''' <summary>
    ''' Toggle the back light power.
    ''' </summary>
    Public Shared Sub ToggleBackLight()
        Static requirementHandle As Integer
        Dim errorSuccess As Integer
        Dim state As PowerState
        'Get back light state
        errorSuccess = GetDevicePower("BKL1:", 1, state)
        If errorSuccess <> 0 Then
            MessageBox.Show("Function not supported by your device.")
            Return
        End If
        If state = DeviceSleep.PowerState.D0 OrElse state = DeviceSleep.PowerState.D1 Then
            'Release hold requirement if any
            If requirementHandle <> 0 Then
                ReleasePowerRequirement(requirementHandle)
            End If
            'This is the recommended way of switching back light off
            requirementHandle = SetPowerRequirement("BKL1:", PowerState.D4, POWER_NAME, IntPtr.Zero, 0)

            'As it doesn't work on all devices, we sometime need to force the back light off
            GetDevicePower("BKL1:", 1, state)
            If state = DeviceSleep.PowerState.D0 OrElse state = DeviceSleep.PowerState.D1 Then
                SetDevicePower("BKL1:", POWER_NAME, PowerState.D4)
            End If
        Else
            'Release hold requirement if any
            If requirementHandle <> 0 Then
                ReleasePowerRequirement(requirementHandle)
            End If
            'This is the recommended way of switching back light on
            requirementHandle = SetPowerRequirement("BKL1:", PowerState.D0, POWER_NAME, IntPtr.Zero, 0)

            'As it doesn't work on all devices, we sometime need to force the back light on.
            'Problem: this could disable the user to put his device in idle state with power button
            GetDevicePower("BKL1:", 1, state)
            If state <> DeviceSleep.PowerState.D0 AndAlso state <> DeviceSleep.PowerState.D1 Then
                SetDevicePower("BKL1:", POWER_NAME, PowerState.D0)
            End If
        End If
        If requirementHandle = 0 Then
            MessageBox.Show("Function not supported by your device.")
        End If
    End Sub
    ''' <summary>
    ''' Switches the screen on. 
    ''' </summary>
    Public Shared Sub PowerOn()
        Dim hdc As IntPtr = GetDC(IntPtr.Zero)

        If ExtEscapeSet(hdc, QUERYESCSUPPORT, 4, SETPOWERMANAGEMENT, 0, IntPtr.Zero) = 0 Then
            MessageBox.Show("Sorry, your device doesn't support software screen power on.")
        Else
            Dim vpm As VideoPowerManagement
            vpm.Initialize()
            'power state
            vpm.PowerState = VideoPowerState.VideoPowerOn
            'Set state
            ExtEscapeSet(hdc, SETPOWERMANAGEMENT, 12, vpm, 0, Nothing)
        End If
        ReleaseDC(IntPtr.Zero, hdc)
    End Sub
End Class
