' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO
Imports System.Threading

Namespace GPS
    Public MustInherit Class SerialPort
        Inherits GPSDevice

        Protected parser As NMEAParser = Nothing

        ' old values
        Protected prevFixState As Boolean
        Protected prevLongitude As Double
        Protected prevLatitude As Double
        Protected prevBearing As Single

        ' private
        Protected commName As String
        Protected baudRate As Integer
        Private myNMEALogFileName As String = ""
        Private myNMEALogFileStream As StreamWriter
        Public Sub New(ByRef parser As NMEAParser, ByVal commName As String, ByVal baudrate As Integer)
            Me.parser = parser
            Me.commName = commName
            Me.baudRate = baudrate
        End Sub
        Public Overrides Function Connect() As Boolean
            gpsConnected = True
            RaiseConnected()
            CreateNMEAlogfile()
            Return True
        End Function
        Public Overrides Sub Disconnect()
            gpsConnected = False
            CloseNMEAlogfile()
            RaiseDisconnected()
        End Sub
        Protected Overrides Sub Finalize()
            CloseNMEAlogfile()
        End Sub
        Public Property NMEALogFileName() As String
            Get
                NMEALogFileName = myNMEALogFileName
            End Get
            Set(ByVal value As String)
                myNMEALogFileName = value
                CreateNMEAlogfile()
            End Set
        End Property
        Protected Sub CreateNMEAlogfile()
            CloseNMEAlogfile()
            If myNMEALogFileName.Length > 0 Then
                myNMEALogFileStream = New StreamWriter(New FileStream(myNMEALogFileName, FileMode.Append))
            End If
        End Sub
        Protected Sub LogNMEA(ByVal data As String)
            If Not myNMEALogFileStream Is Nothing Then
                myNMEALogFileStream.Write(data)
            End If
        End Sub
        Protected Sub CloseNMEAlogfile()
            If Not myNMEALogFileStream Is Nothing Then
                myNMEALogFileStream.Close()
                myNMEALogFileStream = Nothing
            End If
        End Sub
        Protected Sub ReceivedData(ByVal sentence As String)
            lastDataReceivedDate = Now
            If parser.parse(sentence, gpsPosition) = True Then
                If gpsPosition.FixType > 0 Then
                    If prevFixState = False Then
                        prevFixState = True
                        RaiseFixChanged()
                    End If
                    SyncTime(gpsPosition)
                    If gpsPosition.Heading <> prevBearing OrElse gpsPosition.Latitude <> prevLatitude OrElse gpsPosition.Longitude <> prevLongitude Then
                        prevLatitude = gpsPosition.Latitude
                        prevLongitude = gpsPosition.Longitude
                        prevBearing = gpsPosition.Heading
                        RaisePositionChanged()
                    End If
                Else
                    If prevFixState = True Then
                        prevFixState = False
                        RaiseFixChanged()
                    End If
                End If
            End If
        End Sub
    End Class
End Namespace
