' Copyright 2007 Rubke <interrubke@gmail.com>
' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Imports System.Globalization
Imports System.IO
#If Not Win32 Then
Imports Microsoft.WindowsCE.Forms
Imports WaveAudio
#End If

' usefull external links
'http://www.members.tripod.com/~buggerluggs/ie/wav-dir184.htm

' OSMtracker uses two timers:
' - UpdateViewTimer: for updating the titlebar and SpeedGraph
' - TrackLogginTimer: for adding the current data to the gpx file
'
' Apart from this the TileMapPosition and the GPSpanel(tab) is updated on every
' PositionChanged event received from teh GPSDevice-class

Public Class OSMtrackerLogic

    Private OsmTileMap1 As OSMTileMap = Nothing
    Private picTitleBar As PictureTitleBar = Nothing
    Private mainForm As Form = Nothing

    Public Event UpdateGPSInformation(ByVal position As GPS_POSITION)
    Public Event UpdateGUIButtons(ByVal position As GPS_POSITION)
    Public Event UpdateSimulation(ByVal enabled As Boolean)
    Public Event UpdateGPSPosOnMap(ByVal position As GPS_POSITION)
    Public Event UpdateGUIByViewTimer(ByVal position As GPS_POSITION)
    Public Event UpdateDataList()
    Public Event FontSizeChanged(ByVal newSize As Integer)
    Public Event TrackPointAdded(ByVal numOfPoints As Integer)
    Public Event ConfigLoad(ByVal oConfig As XMLConfig)
    Public Event ConfigSave(ByVal oConfig As XMLConfig)

    Private Enum AutoDownloadState As Integer
        Manual = 0
        WithGPRS = 1
        WithGPRSnoRoaming = 2
    End Enum
    Public Shared CultureInfo As New CultureInfo("en-US")

    Private Const NMEA_EXTENTION As String = "nmea"

    Private myPos As GPS_POSITION
    Private oldPos As New GPS_POSITION
    Private WithEvents myGPS As GPS.GPSDevice
    Private myGPXfile As GPXFile = Nothing
    Private myOSMfile As OSMFile = Nothing

    Private storageFolder As String
    Private gpsStoreNMEA As Boolean
    Private gpsCommName As String = ""
    Private gpsBaudrate As Integer = 4800
    Private gpsMetrics As Boolean = True
    Private gpsNoChecksum As Boolean = False
    Private gpxWithDoP As Boolean = False
    Private maxRecDuration As Integer = 2 ' sec
    Private trackLoggingDisabled As Boolean = False
    Private autoPageAdvanceAfterClick As Boolean = True
    Private tileProviders As New TileProviders
    Private soundPlayer As New SoundPlayer
    Private recording As Boolean
    Private recordingBitsPerSecond As Integer
    Private recordingSamplesPerSecond As Integer
    Private replayLastRecording As Boolean
    Private playSoundOnFixChange As Boolean

    Private myFontSize As Integer = 9

    Private bluetoothHandling As Boolean = False
    Private bluetoothWasConnected As Boolean = True
#If Not Win32 Then
    Private WithEvents phoneState As PhoneState
#End If
    Private autoDownload As AutoDownloadState = AutoDownloadState.Manual

    Private WithEvents TrackLoggingTimer As New System.Windows.Forms.Timer
    Private WithEvents UpdateViewTimer As New System.Windows.Forms.Timer

#If PocketPC Then
    Private hwButtons(ButtonXML.HardwareButtons - 1) As HardwareButton
#End If

    Private Delegate Sub AddWayPointThread(ByVal myPos As GPS_POSITION, ByVal time As Date, ByVal Name As String, ByVal comment As String, ByVal link As String)
    Private myAddWayPointDelegate As AddWayPointThread

    Private Delegate Sub RecorderInfoThread(ByVal recState As Boolean)
    Private myRecorderInfoDelegate As RecorderInfoThread

    Private Delegate Sub AddVoiceRecordingThread(ByVal myPos As GPS_POSITION, ByVal time As Date, ByVal Name As String, ByVal comment As String, ByVal link As String)
    Private AddVoiceRecordingDelegate As AddVoiceRecordingThread
    Sub New(ByVal theForm As Form, ByVal osmTileMap As OSMTileMap, ByVal thePicTitleBar As PictureTitleBar)
        mainForm = theForm
        OsmTileMap1 = osmTileMap
        picTitleBar = thePicTitleBar

        theForm.KeyPreview = True

        storageFolder = GetAppPath()

        myPos = New GPS_POSITION
    End Sub
    ''' <summary>
    ''' Setup OSMtrackerLogic, must be called only once directly after instanciating OSMtrackerLogic
    ''' Needed because events cannot be raised in "Sub New"
    ''' </summary>
    Sub AfterCreationSetup()
        LoadSettings()
        ButtonXML.LoadButtons(storageFolder)

#If PocketPC Then
        For i As Integer = 0 To hwButtons.Length - 1
            hwButtons(i) = New HardwareButton()
            hwButtons(i).AssociatedControl = mainForm
        Next
        Try ' try to bind all 6 hwbuttons if needed
            If ButtonXML.HWButtonActions(0) IsNot Nothing Then
                hwButtons(0).HardwareKey = HardwareKeys.ApplicationKey1
            End If
            If ButtonXML.HWButtonActions(1) IsNot Nothing Then
                hwButtons(1).HardwareKey = HardwareKeys.ApplicationKey2
            End If
            If ButtonXML.HWButtonActions(2) IsNot Nothing Then
                hwButtons(2).HardwareKey = HardwareKeys.ApplicationKey3
            End If
            If ButtonXML.HWButtonActions(3) IsNot Nothing Then
                hwButtons(3).HardwareKey = HardwareKeys.ApplicationKey4
            End If
            If ButtonXML.HWButtonActions(4) IsNot Nothing Then
                hwButtons(4).HardwareKey = HardwareKeys.ApplicationKey5
            End If
            If ButtonXML.HWButtonActions(5) IsNot Nothing Then
                hwButtons(5).HardwareKey = HardwareKeys.ApplicationKey6
            End If
        Catch ex As Exception
            ' HWButton not available
        End Try
#End If
        If myGPXfile Is Nothing Then
            myGPXfile = New LoggingGPXFile(storageFolder & IO.Path.DirectorySeparatorChar & GetISOdateString(True), gpxWithDoP)
            ' HACK for DrawTrack on osmTileMap
            OsmTileMap1.GPXfile = myGPXfile
        End If

        ' init osm file
        myOSMfile = New OSMFile(storageFolder & IO.Path.DirectorySeparatorChar & "osm" & IO.Path.DirectorySeparatorChar, GetAppPath() & IO.Path.DirectorySeparatorChar & "img" & IO.Path.DirectorySeparatorChar & "classic.small" & IO.Path.DirectorySeparatorChar)
        OsmTileMap1.OSMfile = myOSMfile

        OsmTileMap1.SetGPSPosition(myPos, True)
        oldPos = myPos

        If myGPS Is Nothing Then
            If bluetoothHandling AndAlso IsWindowsMobile5OrHigher() Then
                If Bluetooth.IsActivated = False Then
                    bluetoothWasConnected = False
                    Bluetooth.Activate()
                    Threading.Thread.Sleep(500)
                End If
            End If
            SetupGPSDevice(gpsCommName)
            myGPS.Connect()
        End If

        myAddWayPointDelegate = New AddWayPointThread(AddressOf AddWayPointThreadMethod)
        myRecorderInfoDelegate = New RecorderInfoThread(AddressOf RecorderInfoThreadMethod)
        AddVoiceRecordingDelegate = New AddVoiceRecordingThread(AddressOf AddVoiceRecordingThreadMethod)

        UpdateViewTimer.Interval = 250
        TrackLoggingTimer.Interval = 1000

        UpdateButtons()
        UpdateGPSpanel()
        UpdateViewTimer.Enabled = True
    End Sub
    Private Function SetupGPSDevice(ByVal type As String) As Boolean
        ' Factory method for GPSDevice
        Select Case type
            Case "simulation"
                Dim parser As New NMEAParser
                parser.noChecksum = gpsNoChecksum
                'select a nmea log file
#If Win32 Then
                Dim dialog As New OpenFileDialog
                dialog.Filter = "NMEA-file|*.nmea"
#Else
                Dim dialog As frmFileFolderSelectDialog = New frmFileFolderSelectDialog("*." & NMEA_EXTENTION)
#End If
                With dialog
                    .Filename = storageFolder
                    If .ShowDialog = Windows.Forms.DialogResult.OK Then
                        myGPS = New GPS.Simulation(parser, .Filename)
                    Else
                        Return False
                    End If
                End With
#If Not Win32 Then
            Case "intermediate", "Intermediate"
                myGPS = New GPS.IntermediateDriver
#End If
            Case Else
                Dim parser As New NMEAParser
                parser.noChecksum = gpsNoChecksum
                Dim serialgps As GPS.SerialPort = Nothing
                If gpsCommName.StartsWith("GPD") Then
#If Not Win32 Then
                    serialgps = New GPS.NativeSerialPort(parser, gpsCommName, gpsBaudrate)
#End If
                Else
                    serialgps = New GPS.DotNetSerialPort(parser, gpsCommName, gpsBaudrate)
                End If
                If gpsStoreNMEA Then
                    serialgps.NMEALogFileName = storageFolder & IO.Path.DirectorySeparatorChar & GetISOdateString(True) & "." & NMEA_EXTENTION
                End If
                myGPS = serialgps
        End Select
        myGPS.Metrics = gpsMetrics
        Return True
    End Function
    Private Sub UpdateGPSpanel()
        RaiseEvent UpdateGPSInformation(myPos)
    End Sub
    Public Sub Connect()
        If gpsCommName.Length > 0 Then
            System.Windows.Forms.Cursor.Current = Cursors.WaitCursor
            myGPS.Connect()
            System.Windows.Forms.Cursor.Current = Cursors.Default
        Else
            MsgBox("Please set up your GPS device first!", MsgBoxStyle.Critical)
        End If
    End Sub
    Public Sub Disconnect()
        myGPS.Disconnect()
        RaiseEvent UpdateSimulation(True)
    End Sub
    Private Sub UpdateViewTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles UpdateViewTimer.Tick
        RaiseEvent UpdateGUIByViewTimer(myPos)
    End Sub
    Private Sub UpdateGPSPosition()
        RaiseEvent UpdateGPSPosOnMap(myPos)
    End Sub
#If Not SmartPhone Then
    Public Sub ShowSettings(ByVal tabPage As String, Optional ByVal noCancel As Boolean = False)
        With frmSettings
            .btnCancel.Enabled = Not noCancel
            .chkEnableSounds.Checked = soundPlayer.IsEnabled
            .chkPlaySoundOnFixChange.Checked = playSoundOnFixChange
            .chkReplayLast.Checked = replayLastRecording
            .chkAutoPageOne.Checked = autoPageAdvanceAfterClick
            .chkDownloadTiles.Checked = OsmTileMap1.DownloadTiles
            .AutoDownload.SelectedIndex = autoDownload
            .lblStorageFolder.Text = storageFolder
            .chkDrawtrack.Checked = OsmTileMap1.DrawTrack
            .chkPaintPOIs.Checked = OsmTileMap1.PaintPOIs
            .chkLogNMEA.Checked = gpsStoreNMEA
            .chkTrackLogging.Checked = Not trackLoggingDisabled
            .chkGPXWithDoP.Checked = gpxWithDoP
            If gpsMetrics Then
                .rbkmh.Checked = True
            Else
                .rbmph.Checked = True
            End If
            .arrowsize.Value = OsmTileMap1.ArrowSize
            .fontsize.Value = myFontSize
            .PopulatecomList(gpsCommName, gpsBaudrate)
            .bluetooth.Checked = bluetoothHandling
            If Not IsWindowsMobile5OrHigher() Then
                .rbIntermediateDriver.Enabled = False
                .bluetooth.Enabled = False
            End If
            If gpsCommName = "Intermediate" AndAlso IsWindowsMobile5OrHigher() Then
                .rbIntermediateDriver.Checked = True
            Else
                .rbIntermediateDriver.Checked = False
            End If

            Dim i As Integer
            For i = 0 To .cmbMaxRecDuration.Items.Count - 1
                If CInt(.cmbMaxRecDuration.Items(i)) = maxRecDuration Then .cmbMaxRecDuration.SelectedIndex = i : Exit For
            Next
            For i = 0 To .cmbSamplesPerSecond.Items.Count - 1
                If CInt(.cmbSamplesPerSecond.Items(i)) = recordingSamplesPerSecond Then .cmbSamplesPerSecond.SelectedIndex = i : Exit For
            Next
            For i = 0 To .cmbBitsPerSecond.Items.Count - 1
                If CInt(.cmbBitsPerSecond.Items(i)) = recordingBitsPerSecond Then .cmbBitsPerSecond.SelectedIndex = i : Exit For
            Next
            For i = 0 To .cmbTrackLogInterval.Items.Count - 1
                If (CInt(.cmbTrackLogInterval.Items(i)) * 1000) = TrackLoggingTimer.Interval Then .cmbTrackLogInterval.SelectedIndex = i : Exit For
            Next

            .trackColorPreview.BackColor = OsmTileMap1.PaintTrackColor
            .waypointColorPreview.BackColor = OsmTileMap1.PaintWaypointColor

            .TileProviders = tileProviders
            .UpdateTileProvidersList()

            ' select tab if given
            Dim tab As TabPage
            For Each tab In .TabControl1.TabPages
                If tab.Text = tabPage Then
                    tab.Show()
                End If
            Next

            If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor
                soundPlayer.IsEnabled = .chkEnableSounds.Checked
                playSoundOnFixChange = .chkPlaySoundOnFixChange.Checked
                replayLastRecording = .chkReplayLast.Checked
                autoPageAdvanceAfterClick = .chkAutoPageOne.Checked
                maxRecDuration = Integer.Parse(.cmbMaxRecDuration.Text)
                recordingBitsPerSecond = Integer.Parse(.cmbBitsPerSecond.Text)
                recordingSamplesPerSecond = Integer.Parse(.cmbSamplesPerSecond.Text)
                trackLoggingDisabled = True ' disable logging now until all settings are applied
                gpxWithDoP = .chkGPXWithDoP.Checked
                If Integer.Parse(.cmbTrackLogInterval.Text) > 0 Then
                    TrackLoggingTimer.Interval = Integer.Parse(.cmbTrackLogInterval.Text) * 1000 ' make milsecondes
                End If
                OsmTileMap1.PaintTrackColor = .trackColorPreview.BackColor
                OsmTileMap1.PaintWaypointColor = .waypointColorPreview.BackColor
                OsmTileMap1.DrawTrack = .chkDrawtrack.Checked
                OsmTileMap1.PaintPOIs = .chkPaintPOIs.Checked
                gpsStoreNMEA = .chkLogNMEA.Checked
                autoDownload = CType(.AutoDownload.SelectedIndex, AutoDownloadState)
                If autoDownload > 0 Then
                    SetAutoDownload()
                End If
                OsmTileMap1.DownloadTiles = .chkDownloadTiles.Checked
                gpsMetrics = .rbkmh.Checked
                OsmTileMap1.ArrowSize = CInt(.arrowsize.Value)
                myFontSize = CInt(.fontsize.Value)
                RaiseEvent FontSizeChanged(myFontSize)
                bluetoothHandling = .bluetooth.Checked

                If storageFolder <> .lblStorageFolder.Text Then
                    storageFolder = .lblStorageFolder.Text
                    CheckFolders()
                    NewGPXFile()
                End If

                If .rbIntermediateDriver.Checked = True Then
                    gpsCommName = "Intermediate"
                End If

                ' prevent disconnect and connect if no GPS setting changed
                If (gpsCommName <> .CommName) Or (gpsBaudrate <> .Baudrate) Or .btnCancel.Enabled = False Then
                    If myGPS IsNot Nothing Then
                        myGPS.Disconnect()
                    End If
                    If .rbSerialDriver.Checked Then
                        gpsCommName = .CommName
                        gpsBaudrate = .Baudrate
                    End If
                    SetupGPSDevice(gpsCommName)
                    myGPS.Connect()
                End If
                myGPS.Metrics = gpsMetrics

                trackLoggingDisabled = Not .chkTrackLogging.Checked
                UpdateTrackLogging()

                'store setting immedately
                StoreSettings()

                tileProviders.GotoProvider(0)
                OsmTileMap1.UpdateTileProvider(tileProviders.GetCurrentTileProvider)

                System.Windows.Forms.Cursor.Current = Cursors.Default
            End If
        End With
    End Sub
#Else
    Public Sub ShowSettingsGeneral()
        With SettingsGeneral
            .chkEnableSounds.Checked = SoundPlayer.IsEnabled
            .chkPlaySoundOnFixChange.Checked = playSoundOnFixChange
            .chkAutoPageOne.Checked = autoPageAdvanceAfterClick
            .lblStorageFolder.Text = storageFolder
            If gpsMetrics Then
                .measuringunits.SelectedIndex = 0
            Else
                .measuringunits.SelectedIndex = 1
            End If
            If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

                SoundPlayer.IsEnabled = .chkEnableSounds.Checked
                playSoundOnFixChange = .chkPlaySoundOnFixChange.Checked
                autoPageAdvanceAfterClick = .chkAutoPageOne.Checked
                gpsMetrics = (.measuringunits.SelectedIndex = 0)

                myGPS.Metrics = gpsMetrics

                If storageFolder <> .lblStorageFolder.Text Then
                    storageFolder = .lblStorageFolder.Text
                    CheckFolders()
                    NewGPXFile()
                End If

                'store setting immedately
                StoreSettings()

                System.Windows.Forms.Cursor.Current = Cursors.Default
            End If
        End With
    End Sub
    Public Sub ShowSettingsGPS()
        With SettingsGPS
            .bluetooth.Checked = bluetoothHandling
            .PopulatecomList(gpsCommName, gpsBaudrate)

            .drivermodell.SelectedIndex = 0
            If gpsCommName <> "Intermediate" Then
                .drivermodell.SelectedIndex = 1
            End If

            If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

                bluetoothHandling = .bluetooth.Checked

                gpsCommName = .drivermodell.Text

                ' prevent disconnect and connect if no GPS setting changed
                If (gpsCommName <> .CommName) Or (gpsBaudrate <> .Baudrate) Then
                    If myGPS IsNot Nothing Then
                        myGPS.Disconnect()
                    End If
                    If gpsCommName <> "Intermediate" Then
                        gpsCommName = .CommName
                        gpsBaudrate = .Baudrate
                    End If
                    SetupGPSDevice(gpsCommName)
                    myGPS.Connect()
                End If

                'store setting immedately
                StoreSettings()

                System.Windows.Forms.Cursor.Current = Cursors.Default
            End If
        End With
    End Sub
    Public Sub ShowSettingsVoice()
        With SettingsVoice
            .chkReplayLast.Checked = replayLastRecording

            Dim i As Integer
            For i = 0 To .cmbMaxRecDuration.Items.Count - 1
                If CInt(.cmbMaxRecDuration.Items(i)) = maxRecDuration Then .cmbMaxRecDuration.SelectedIndex = i : Exit For
            Next
            For i = 0 To .cmbSamplesPerSecond.Items.Count - 1
                If CInt(.cmbSamplesPerSecond.Items(i)) = recordingSamplesPerSecond Then .cmbSamplesPerSecond.SelectedIndex = i : Exit For
            Next
            For i = 0 To .cmbBitsPerSecond.Items.Count - 1
                If CInt(.cmbBitsPerSecond.Items(i)) = recordingBitsPerSecond Then .cmbBitsPerSecond.SelectedIndex = i : Exit For
            Next

            If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

                replayLastRecording = .chkReplayLast.Checked
                maxRecDuration = Integer.Parse(.cmbMaxRecDuration.Text)
                recordingBitsPerSecond = Integer.Parse(.cmbBitsPerSecond.Text)
                recordingSamplesPerSecond = Integer.Parse(.cmbSamplesPerSecond.Text)

                'store setting immedately
                StoreSettings()

                System.Windows.Forms.Cursor.Current = Cursors.Default
            End If
        End With
    End Sub
    Public Sub ShowSettingsLogging()
        With SettingsLogging
            .chkTrackLogging.Checked = Not trackLoggingDisabled
            .chkGPXWithDoP.Checked = gpxWithDoP
            .chkLogNMEA.Enabled = gpsCommName <> "Intermediate"
            If .chkLogNMEA.Enabled Then
                .chkLogNMEA.Checked = gpsStoreNMEA
            Else
                .chkLogNMEA.Checked = False
            End If

            Dim i As Integer
            For i = 0 To .cmbTrackLogInterval.Items.Count - 1
                If (CInt(.cmbTrackLogInterval.Items(i)) * 1000) = TrackLoggingTimer.Interval Then .cmbTrackLogInterval.SelectedIndex = i : Exit For
            Next

            If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

                trackLoggingDisabled = True ' disable logging now until all settings are applied
                gpxWithDoP = .chkGPXWithDoP.Checked
                TrackLoggingTimer.Interval = Integer.Parse(.cmbTrackLogInterval.Text) * 1000 ' make milsecondes
                gpsStoreNMEA = .chkLogNMEA.Checked

                trackLoggingDisabled = Not .chkTrackLogging.Checked
                UpdateTrackLogging()

                'store setting immedately
                StoreSettings()

                System.Windows.Forms.Cursor.Current = Cursors.Default
            End If
        End With
    End Sub
    Public Sub ShowSettingsMap()
        With SettingsMap
            .arrowsize.Text = OsmTileMap1.ArrowSize.ToString

            .chkDownloadTiles.Checked = OsmTileMap1.DownloadTiles
            .AutoDownload.SelectedIndex = autoDownload

            .chkDrawtrack.Checked = OsmTileMap1.DrawTrack
            .chkPaintPOIs.Checked = OsmTileMap1.PaintPOIs

            .trackColorPreview.BackColor = OsmTileMap1.PaintTrackColor
            .waypointColorPreview.BackColor = OsmTileMap1.PaintWaypointColor

            .TileProviders = tileProviders
            .UpdateTileProvidersList()

            If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

                OsmTileMap1.PaintTrackColor = .trackColorPreview.BackColor
                OsmTileMap1.PaintWaypointColor = .waypointColorPreview.BackColor
                OsmTileMap1.DrawTrack = .chkDrawtrack.Checked
                OsmTileMap1.PaintPOIs = .chkPaintPOIs.Checked

                autoDownload = CType(.AutoDownload.SelectedIndex, AutoDownloadState)
                If autoDownload > 0 Then
                    SetAutoDownload()
                End If
                OsmTileMap1.DownloadTiles = .chkDownloadTiles.Checked

                OsmTileMap1.ArrowSize = CInt(.arrowsize.Text)

                tileProviders.GotoProvider(0)
                OsmTileMap1.UpdateTileProvider(tileProviders.GetCurrentTileProvider)

                System.Windows.Forms.Cursor.Current = Cursors.Default
            End If
        End With
    End Sub
#End If
    Private Sub myGPS_Connected() Handles myGPS.Connected
        RaiseEvent UpdateSimulation(False)
        UpdateViewTimer.Enabled = True
        UpdateButtons()
        picTitleBar.UpdateData(myGPS.HasFix, myGPS.IsConnected, myGPS.SatellitesInUse, myGPS.Speed, myGPS.Bearing, Not trackLoggingDisabled)
        myGPXfile.AddTrack()
        UpdateTrackLogging()
        ' when GPS connected- prevent device to go to sleep mode
#If Not Win32 Then
        DeviceSleep.PreventSleepMode(True)
#End If
    End Sub
    Private Sub myGPS_DisConnected() Handles myGPS.Disconnected
        ' when GPS connected- turn off sleep mode prevention
#If Not Win32 Then
        DeviceSleep.PreventSleepMode(False)
#End If
        picTitleBar.UpdateData(myGPS.HasFix, myGPS.IsConnected, myGPS.SatellitesInUse, myGPS.Speed, myGPS.Bearing, Not trackLoggingDisabled)
        RaiseEvent UpdateSimulation(True)
        ' set driver back to the driver choosen in settings dialog if we were in simulation mode
        If myGPS.GetType.Name = "Simulation" And gpsCommName <> "" Then
            SetupGPSDevice(gpsCommName)
        End If
    End Sub
    Private Sub FillPosListView()
        If myGPXfile Is Nothing Then Exit Sub
        RaiseEvent UpdateDataList()
    End Sub
    Private Sub LoadSettings()
        Dim oConfig As New XMLConfig(GetAppPath() & IO.Path.DirectorySeparatorChar & "config.xml")
        soundPlayer.IsEnabled = oConfig.GetBooleanSetting("EnableSounds", True)
        replayLastRecording = oConfig.GetBooleanSetting("ReplayLast", False)
        autoPageAdvanceAfterClick = oConfig.GetBooleanSetting("AutoPageOneAfterClick", True)
        gpsStoreNMEA = oConfig.GetBooleanSetting("StoreNMEA", False)
#If Not Win32 Then
        storageFolder = oConfig.GetSetting("StorageFolder", "\My Documents\OSMtracker")
#Else
        storageFolder = oConfig.GetSetting("StorageFolder", My.Computer.FileSystem.SpecialDirectories.MyDocuments & IO.Path.DirectorySeparatorChar &"OSMtracker")
#End If
        gpsNoChecksum = oConfig.GetBooleanSetting("NoGPSChecksumVerify", False)
        bluetoothHandling = oConfig.GetBooleanSetting("BluetoothHandling", False) AndAlso IsWindowsMobile5OrHigher()
        gpsCommName = oConfig.GetSetting("CommName", "Intermediate")
        gpsBaudrate = oConfig.GetIntegerSetting("CommBaudrate", 4800)
        maxRecDuration = oConfig.GetIntegerSetting("MaxRecDuration", 5)
        recordingBitsPerSecond = oConfig.GetIntegerSetting("BitsPerSecond", 8)
        recordingSamplesPerSecond = oConfig.GetIntegerSetting("SamplesPerSecond", 22050)
        trackLoggingDisabled = oConfig.GetBooleanSetting("TrackLoggingDisabled", False)
        gpxWithDoP = oConfig.GetBooleanSetting("GPXStoreDoP", False)
        TrackLoggingTimer.Interval = CInt(oConfig.GetDoubleSetting("TrackLoggingInterval", 2) * 1000)
        If oConfig.GetIntegerSetting("AutoDownloadTiles", 0) > 0 Then
            autoDownload = CType(oConfig.GetIntegerSetting("AutoDownloadTiles", 0), AutoDownloadState)
            SetAutoDownload()
        Else
            OsmTileMap1.DownloadTiles = oConfig.GetBooleanSetting("DownloadTiles", False)
        End If
        OsmTileMap1.PaintTrackColor = oConfig.GetColorSetting("TrackColor", Color.Orange)
        OsmTileMap1.PaintWaypointColor = oConfig.GetColorSetting("WaypointColor", Color.Blue)
        OsmTileMap1.DrawTrack = oConfig.GetBooleanSetting("DrawTrackOnMap", False)
        OsmTileMap1.PaintPOIs = oConfig.GetBooleanSetting("PaintPOIsOnMap", False)
        OsmTileMap1.ArrowSize = oConfig.GetIntegerSetting("ArrowSize", 14)
        gpsMetrics = oConfig.GetBooleanSetting("MetricsEurope", True)
        playSoundOnFixChange = oConfig.GetBooleanSetting("PlaySoundOnFixChange", False)
        myPos.Change(oConfig.GetDoubleSetting("LastLatitude", 0), oConfig.GetDoubleSetting("LastLongitude", 0))
        UpdateTrackLogging()
        If oConfig.GetBooleanSetting("FullScreen", True) = False Then
            mainForm.WindowState = FormWindowState.Normal
        End If
        myFontSize = oConfig.GetIntegerSetting("FontSize", 9)
        RaiseEvent FontSizeChanged(myFontSize)

        CheckFolders()

        RaiseEvent ConfigLoad(oConfig)

        ' Smoothly mirgrate "old" settings
        If oConfig.GetSetting("TileStorageFolder") <> "" Then
            oConfig.SetSetting("TileProviderName1", "imported provider")
            oConfig.SetSetting("TileStorageFolder1", oConfig.GetSetting("TileStorageFolder", storageFolder & IO.Path.DirectorySeparatorChar & "Tiles"))
            oConfig.DeleteSetting("TileStorageFolder")
        End If
        If oConfig.GetSetting("CurrentTileServerUrl") <> "" Then
            oConfig.SetSetting("TileServerUrl1", oConfig.GetSetting("CurrentTileServerUrl"))
            oConfig.DeleteSetting("CurrentTileServerUrl")
        End If
        If oConfig.GetSetting("TileProviderName1") = "" Then
            If MsgBox("There is no tile provider defined. Do you want to set up Mapnik?", MsgBoxStyle.YesNo, "OSMtracker") = MsgBoxResult.Yes Then
                oConfig.SetSetting("TileProviderName1", "Mapnik")
                oConfig.SetSetting("TileStorageFolder1", storageFolder & IO.Path.DirectorySeparatorChar & "Tiles")
                oConfig.SetSetting("TileServerUrl1", "http://tile.openstreetmap.org/mapnik/")
                oConfig.SetSetting("TileServerMinZoom1", MIN_ZOOM)
                oConfig.SetSetting("TileServerMaxZoom1", MAX_ZOOM)
                MsgBox("If you want to download tiles automatically, enable this option on the settings dialog.")
            End If
        End If
        Dim TileServerFound As Boolean = True
        Dim i As Integer = 1
        While TileServerFound
            Dim strTileProviderName As String = oConfig.GetSetting("TileProviderName" & i)
            If strTileProviderName.Length <> 0 Then
                tileProviders.AddTileProvider(strTileProviderName, oConfig.GetSetting("TileStorageFolder" & i), oConfig.GetSetting("TileServerUrl" & i), oConfig.GetIntegerSetting("TileServerMinZoom" & i, MIN_ZOOM), oConfig.GetIntegerSetting("TileServerMaxZoom" & i, MAX_ZOOM))
            Else
                TileServerFound = False
            End If
            i += 1
        End While
        OsmTileMap1.UpdateTileProvider(tileProviders.GetCurrentTileProvider)
        OsmTileMap1.CurrentZoom = oConfig.GetIntegerSetting("LastZoom", 13)   ' last zoom 
        If Not isDevelopmentVersion() AndAlso oConfig.GetSetting("Version", "") <> getAppVersion() Then
            'ShowSettings("", True)
        End If
    End Sub
    Private Sub CheckFolders()
        If Not IO.Directory.Exists(storageFolder) Then
            MsgBox("Storage folder missing, trying to create" & storageFolder, MsgBoxStyle.Information)
            Try
                IO.Directory.CreateDirectory(storageFolder)
            Catch ex As Exception
                MsgBox("Error creating folder: " & ex.Message, MsgBoxStyle.Critical)
                storageFolder = GetAppPath()
            End Try
        End If
    End Sub
    Private Sub StoreSettings()
        Dim oConfig As New XMLConfig(GetAppPath() & IO.Path.DirectorySeparatorChar & "config.xml")
        oConfig.SetSetting("Version", getAppVersion())
        oConfig.SetSetting("EnableSounds", soundPlayer.IsEnabled)
        oConfig.SetSetting("ReplayLast", replayLastRecording)
        oConfig.SetSetting("AutoPageOneAfterClick", autoPageAdvanceAfterClick)
        oConfig.SetSetting("StoreNMEA", gpsStoreNMEA)
        oConfig.SetSetting("StorageFolder", storageFolder)
        oConfig.SetSetting("DownloadTiles", OsmTileMap1.DownloadTiles And autoDownload = AutoDownloadState.Manual)
        oConfig.SetSetting("AutoDownloadTiles", autoDownload)
        oConfig.SetSetting("DrawTrackOnMap", OsmTileMap1.DrawTrack)
        oConfig.SetSetting("PaintPOIsOnMap", OsmTileMap1.PaintPOIs)
        oConfig.SetSetting("TrackColor", OsmTileMap1.PaintTrackColor)
        oConfig.SetSetting("WaypointColor", OsmTileMap1.PaintWaypointColor)
        oConfig.SetSetting("BluetoothHandling", bluetoothHandling)
        oConfig.SetSetting("CommName", gpsCommName)
        oConfig.SetSetting("CommBaudrate", gpsBaudrate)
        oConfig.SetSetting("MaxRecDuration", maxRecDuration)
        oConfig.SetSetting("BitsPerSecond", recordingBitsPerSecond)
        oConfig.SetSetting("SamplesPerSecond", recordingSamplesPerSecond)
        oConfig.SetSetting("TrackLoggingDisabled", trackLoggingDisabled)
        oConfig.SetSetting("TrackLoggingInterval", TrackLoggingTimer.Interval / 1000) ' to sec
        oConfig.SetSetting("MetricsEurope", gpsMetrics)
        oConfig.SetSetting("LastLongitude", myPos.Longitude)
        oConfig.SetSetting("LastLatitude", myPos.Latitude)
        oConfig.SetSetting("PlaySoundOnFixChange", playSoundOnFixChange)
        oConfig.SetSetting("LastZoom", OsmTileMap1.CurrentZoom)
        oConfig.SetSetting("ArrowSize", OsmTileMap1.ArrowSize)
        oConfig.SetSetting("GPXStoreDoP", gpxWithDoP)
        oConfig.SetSetting("FontSize", myFontSize)
        oConfig.SetSetting("FullScreen", Not mainForm.WindowState = FormWindowState.Normal)
        RaiseEvent ConfigSave(oConfig)

        ' store tile providers
        Dim i As Integer
        For i = 1 To tileProviders.Count
            tileProviders.GotoProvider(i - 1)
            oConfig.SetSetting("TileProviderName" & i, tileProviders.GetCurrentName)
            oConfig.SetSetting("TileStorageFolder" & i, tileProviders.GetCurrentFolderName)
            oConfig.SetSetting("TileServerUrl" & i, tileProviders.GetCurrentURL)
            oConfig.SetSetting("TileServerMinZoom" & i, tileProviders.GetCurrentMinZoom)
            oConfig.SetSetting("TileServerMaxZoom" & i, tileProviders.GetCurrentMaxZoom)
        Next
        ' remove deleted tile providers
        Dim oldTileProviderFound As Boolean = True
        While oldTileProviderFound
            If oConfig.GetSetting("TileProviderName" & i).Length <> 0 Then
                oConfig.DeleteSetting("TileProviderName" & i)
                oConfig.DeleteSetting("TileStorageFolder" & i)
                oConfig.DeleteSetting("TileServerUrl" & i)
                oConfig.DeleteSetting("TileServerMinZoom" & i)
                oConfig.DeleteSetting("TileServerMaxZoom" & i)
            Else
                oldTileProviderFound = False
            End If
            i += 1
        End While
    End Sub
    Public Sub ShuttingDown()
        ' only ensure that SleepMode prevention is off
#If Not Win32 Then
        If DeviceSleep.IsPreventionOn() Then
            DeviceSleep.PreventSleepMode(False)
        End If
#End If
        UpdateViewTimer.Enabled = False
        TrackLoggingTimer.Enabled = False
        myGPS.Disconnect()
        myGPXfile.Save()
        StoreSettings()
        If bluetoothHandling = True AndAlso IsWindowsMobile5OrHigher() Then
            If bluetoothWasConnected Then
                If MsgBox("Bluetooth was activated on OSMtracker startup. Shall it be deactivated now?", MsgBoxStyle.YesNo) = vbYes Then
                    Bluetooth.Deactivate()
                End If
            Else
                Bluetooth.Deactivate()
            End If
        End If
        OsmTileMap1.Shutdown()
    End Sub
    Private Function GetISOdateString(ByVal safefile As Boolean) As String
        If safefile Then
            GetISOdateString = Now.ToString("yyyyMMdd_HHmmss", CultureInfo)
        Else
            GetISOdateString = Now.ToString("yyyy-MM-ddTHH:mm.ssZ", CultureInfo)
        End If
    End Function
    Public Sub DeleteWaypoint(ByVal index As Integer, ByVal fileName As String)
        If myGPXfile.RemoveWayPoint(index) Then
            ' TODO HACK, move to GPXFile
            If fileName <> "" AndAlso File.Exists(fileName) Then
                File.Delete(fileName)
            End If
        End If
    End Sub
    Private Sub UpdateButtons()
        RaiseEvent UpdateGUIButtons(myPos)
    End Sub
    Private Sub AddVoiceRecordingThreadMethod(ByVal position As GPS_POSITION, ByVal time As Date, ByVal Name As String, ByVal comment As String, ByVal link As String)
        AddWayPoint(position, time, Name, comment, link)
        FillPosListView()
        If replayLastRecording = True Then
            If link.Length > 0 And link.EndsWith(".wav") Then
                picTitleBar.SetMessage2("Replaying...", Color.LightGreen, maxRecDuration)
                Application.DoEvents()
                Dim strFileName As String = storageFolder & IO.Path.DirectorySeparatorChar & link
                soundPlayer.PlayClip(strFileName)
            End If
        End If
    End Sub
    Private Sub AddWayPointThreadMethod(ByVal position As GPS_POSITION, ByVal time As Date, ByVal Name As String, ByVal comment As String, ByVal link As String)
        AddWayPoint(position, time, Name, comment, link)
        FillPosListView()
    End Sub
    Public Function AddWayPoint(ByVal position As GPS_POSITION, ByVal time As Date, ByVal Name As String, ByVal comment As String, ByVal link As String) As Boolean
        '<wpt lat="50.0" lon="5.0">
        ' <ele>116.204231</ele>
        ' <time>2007-07-22T21:23:21.500Z</time>
        ' <name>audio1</name>
        ' <link href="20070723 222648.wav" />
        ' <desc>001</desc>
        '</wpt>

        myGPXfile.AddWayPoint(position, time, Name, comment, link)
        OsmTileMap1.PaintWayPoint()
    End Function
    Public Sub SyncTime()
        ' set trigger for next time received event
        myGPS.RequestTimeSync()
    End Sub
    Private Sub UpdateTrackLogging()
        TrackLoggingTimer.Enabled = (trackLoggingDisabled = False AndAlso TrackLoggingTimer.Interval <> 0)
    End Sub
    Private Sub TrackLoggingTimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TrackLoggingTimer.Tick
        LogTrackPoint()
    End Sub
    Public Sub NewGPXFile()
        Dim gpxName As String = ""
        While gpxName = ""
            Dim inputbox As New frmInputBox(prompt:="Enter gpx filename here:", defaultResponse:=GetISOdateString(True))
            Dim result As DialogResult = inputbox.ShowDialog()
            If result = System.Windows.Forms.DialogResult.OK Then
                If Not File.Exists(inputbox.GetResult) Then
                    gpxName = inputbox.GetResult
                Else
                    gpxName = ""
                End If
            ElseIf result = DialogResult.Cancel Then
                Exit Sub
            End If
        End While
        Dim oldTrackLoggingDisabled As Boolean = trackLoggingDisabled
        trackLoggingDisabled = True
        UpdateTrackLogging()
        If myGPXfile IsNot Nothing Then
            myGPXfile.Save()
        End If
        myGPXfile = New LoggingGPXFile(storageFolder & IO.Path.DirectorySeparatorChar & gpxName, gpxWithDoP)
        'need to update the POI list, because a new GPX file has a new (empty) waypoint list
        FillPosListView()
        ' HACK for DrawTrack on osmTileMap
        OsmTileMap1.GPXfile = myGPXfile
        trackLoggingDisabled = oldTrackLoggingDisabled
        UpdateTrackLogging()
    End Sub
    Public Sub OpenRouteGPXFile(ByVal openRouteGPX As String)
        If (myGPXfile.GetTrackPointsCount > 0 OrElse myGPXfile.GetWayPointsCount > 0) AndAlso MsgBox("Do you also want to start a new track/waypoint logging gpx-file? Otherwise the current file will be kept.", MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            NewGPXFile()
        End If
        myGPXfile = New RouteGPXFile(openRouteGPX, gpxWithDoP, myGPXfile)
        ' HACK for DrawTrack on osmTileMap
        OsmTileMap1.GPXfile = myGPXfile
    End Sub
    Private Sub LogTrackPoint()
        If Not myGPS Is Nothing Then
            If myGPS.HasFix AndAlso (myPos <> oldPos) Then
                myGPXfile.AddTrackPoint(myPos, Now)
                oldPos = myPos
                RaiseEvent TrackPointAdded(myGPXfile.GetTrackPointsCount)
            End If
        End If
    End Sub
    Public Sub HandleButtonAction(ByRef button As Button)
        Dim command As String, text As String, gotoPage As Integer = -1, SoundFile As String

        command = button.Tag
        text = button.Title
        gotoPage = button.GotoPage
        SoundFile = button.SoundFile
        Select Case command.ToLower
            Case "__next"
                If (ButtonXML.CurrentPageID < ButtonXML.Pages.Count) Then
                    HandleGotoPage(ButtonXML.CurrentPageID + 1, True, SoundFile)
                Else
                    HandleGotoPage(1, True, SoundFile)
                End If

            Case "__prev"
                If (ButtonXML.CurrentPageID > 1) Then
                    HandleGotoPage(ButtonXML.CurrentPageID - 1, True, SoundFile)
                Else
                    HandleGotoPage(ButtonXML.Pages.Count, True, SoundFile)
                End If

            Case "_synctime"
                soundPlayer.PlayClick(SoundFile)
                myGPS.RequestTimeSync()
                HandleAutoPageAdvance()

            Case "__downloadtiles"
                soundPlayer.PlayClick(SoundFile)
                OsmTileMap1.DownloadTiles = Not OsmTileMap1.DownloadTiles
                If OsmTileMap1.DownloadTiles Then
                    picTitleBar.SetMessage2("Download is ON", Color.LightYellow, 3)
                Else
                    picTitleBar.SetMessage2("Download is OFF", Color.LightGray, 3)
                End If
                'handleAutoPageAdvance()

            Case "__goto"
                HandleGotoPage(gotoPage, True, SoundFile)

#If Not SmartPhone Then
            Case "__settings"
                soundPlayer.PlayClick(SoundFile)
                ShowSettings("")
                'handleAutoPageAdvance()
#End If

            Case "__exit"
                soundPlayer.PlayClick(SoundFile)
                mainForm.Close()

            Case "__zoomin"
                soundPlayer.PlayClick(SoundFile)
                OsmTileMap1.ZoomIn()
                HandleAutoPageAdvance()

            Case "__zoomout"
                soundPlayer.PlayClick(SoundFile)
                OsmTileMap1.ZoomOut()
                HandleAutoPageAdvance()

            Case "__centertocurrentposition"
                soundPlayer.PlayClick(SoundFile)
                OsmTileMap1.ExitManualMode()

            Case "__fullscreen"
                soundPlayer.PlayClick(SoundFile)
                If mainForm.WindowState = FormWindowState.Maximized Then
                    mainForm.WindowState = FormWindowState.Normal
                Else
                    mainForm.WindowState = FormWindowState.Maximized
                End If
                HandleAutoPageAdvance()

#If Not Win32 Then
            Case "_takepicture"
                soundPlayer.PlayClick(SoundFile)
                Dim takePictureThread As New Threading.Thread(New Threading.ThreadStart(AddressOf Me.TakePicture))
                takePictureThread.Start() ' //take picture in new thread
                HandleAutoPageAdvance()
#End If

            Case "_voicerec"
                If recording = False Then
                    recording = True    ' prevent another recording
                    Dim recordVoiceThread As New Threading.Thread(New Threading.ThreadStart(AddressOf Me.RecordVoice))
                    recordVoiceThread.Start() ' //record in new thread
                    HandleAutoPageAdvance()
                End If

            Case "__tracking"
                soundPlayer.PlayClick(SoundFile)
                If trackLoggingDisabled = True Then
                    myGPXfile.AddTrack()
                    picTitleBar.SetMessage2("Track logging ON ", Color.LightYellow, 3)
                Else
                    picTitleBar.SetMessage2("Track logging OFF", Color.LightGray, 3)
                End If
                trackLoggingDisabled = Not trackLoggingDisabled
                UpdateTrackLogging()

            Case "_customtext"
                soundPlayer.PlayClick(SoundFile)
                Dim position As GPS_POSITION = myPos
                Dim inputbox As New frmInputBox("Enter your waypoint here:")
                If inputbox.ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                    Dim customtext As String = inputbox.GetResult
                    If customtext <> "" Then
                        AddWayPoint(position, Now, customtext, "", "")
                        FillPosListView()
                    End If
                End If
                HandleAutoPageAdvance()

            Case "__nexttileprovider"
                soundPlayer.PlayClick(SoundFile)
                tileProviders.NextProvider()
                picTitleBar.SetMessage2("Tile provider: " & tileProviders.GetCurrentTileProvider.Name, Color.LightGreen, 3)
                OsmTileMap1.UpdateTileProvider(tileProviders.GetCurrentTileProvider, True)
                HandleAutoPageAdvance()

            Case "__newgpx"
                soundPlayer.PlayClick(SoundFile)
                NewGPXFile()
                picTitleBar.SetMessage2("New GPX-file started...", Color.LightGreen, 3)

            Case "__newtrack"
                soundPlayer.PlayClick(SoundFile)
                myGPXfile.AddTrack()
                picTitleBar.SetMessage2("New track started...", Color.LightGreen, 3)

            Case "__drawtrack"
                soundPlayer.PlayClick(SoundFile)
                OsmTileMap1.DrawTrack = Not OsmTileMap1.DrawTrack
                If OsmTileMap1.DrawTrack Then
                    picTitleBar.SetMessage2("Draw track is ON", Color.LightYellow, 3)
                Else
                    picTitleBar.SetMessage2("Draw track is OFF", Color.LightGray, 3)
                End If

            Case "__about"
                soundPlayer.PlayClick(SoundFile)
                frmAbout.ShowDialog()
                'handleAutoPageAdvance()

                ' TODO 2bd
#If HAVE_TABS Then
            Case "__map"
                soundPlayer.PlayClick(SoundFile)
                TabControl1.SelectedIndex = TabNames.Map

            Case "__gps"
                soundPlayer.PlayClick(SoundFile)
                TabControl1.SelectedIndex = TabNames.GPS

            Case "__waypoints"
                soundPlayer.PlayClick(SoundFile)
                TabControl1.SelectedIndex = TabNames.Waypoints

            Case "__data"
                soundPlayer.PlayClick(SoundFile)
                TabControl1.SelectedIndex = TabNames.Data

            Case "__nexttab"
                soundPlayer.PlayClick(SoundFile)
                If TabControl1.SelectedIndex < TabControl1.TabPages.Count - 1 Then
                    TabControl1.SelectedIndex = TabControl1.SelectedIndex + 1
                Else
                    TabControl1.SelectedIndex = 0
                End If

            Case "__prevtab"
                soundPlayer.PlayClick(SoundFile)
                If TabControl1.SelectedIndex > 1 Then
                    TabControl1.SelectedIndex = TabControl1.SelectedIndex - 1
                Else
                    TabControl1.SelectedIndex = TabControl1.TabPages.Count - 1
                End If
#End If

#If Not Win32 Then
            Case "__screenoff"
                soundPlayer.PlayClick(SoundFile)
                DeviceSleep.PowerOff()

            Case "__backlight"
                soundPlayer.PlayClick(SoundFile)
                DeviceSleep.ToggleBackLight()
#End If

            Case "__loadbtnfile"
                ButtonXML.LoadButtons(storageFolder, button.ExecPath)
                HandleGotoPage(1, True, SoundFile)

            Case "__exec"
                Dim ERROR_FILE_NOT_FOUND As Integer = 2
                Dim ERROR_ACCESS_DENIED As Integer = 5
                Dim startInfo As New ProcessStartInfo

                Try
                    startInfo.FileName = GetAppPath() & IO.Path.DirectorySeparatorChar & button.ExecPath
                    startInfo.WorkingDirectory = GetAppPath()

                    Process.Start(startInfo)

                    startInfo.Arguments = ""

                    Process.Start(startInfo)
                Catch e As System.ComponentModel.Win32Exception
                    If e.NativeErrorCode = ERROR_FILE_NOT_FOUND Then
                        MsgBox(e.Message + ". Check the path.")

                    ElseIf e.NativeErrorCode = ERROR_ACCESS_DENIED Then
                        MsgBox(e.Message + ". You do not have permission to start this file.")
                    Else
                        MsgBox(e.Message & ": " & e.NativeErrorCode.ToString & " " & startInfo.FileName)
                    End If
                End Try
            Case Else ' a waypoint button
                soundPlayer.PlayClick(SoundFile)
                If text.Length > 0 Then
                    AddWayPoint(myPos, Now, text.Replace(vbCrLf, " "), "", "")
                    FillPosListView()
                End If
                HandleAutoPageAdvance()
        End Select
    End Sub
    Private Sub HandleHwButton(ByVal button As Integer)
        Dim btn As Button = ButtonXML.HWButtonActions(button - 1)
        If btn IsNot Nothing Then
            HandleButtonAction(btn)
        End If
    End Sub
    Private Sub HandleAutoPageAdvance(Optional ByVal withSound As Boolean = False, Optional ByVal soundFile As String = "")
        If autoPageAdvanceAfterClick Then
            If ButtonXML.Pages.ContainsKey(ButtonXML.CurrentPageID) Then
                Dim Page As Page = CType(ButtonXML.Pages.Item(ButtonXML.CurrentPageID), Page)
                HandleGotoPage(Page.GotoPage, withSound, soundFile)
            End If
        End If
    End Sub
    Private Sub HandleGotoPage(ByVal pageNumber As Integer, Optional ByVal withSound As Boolean = False, Optional ByVal soundFile As String = "")
        If pageNumber > 0 Then
            If (ButtonXML.Pages.ContainsKey(pageNumber)) Then
                ButtonXML.CurrentPageID = pageNumber
                If withSound Then soundPlayer.PlayClick(soundFile)
                RaiseEvent UpdateGUIButtons(myPos)
            Else
                soundPlayer.PlayBlip(soundFile)
                picTitleBar.SetMessage2("Page " & pageNumber & " is invalid", Color.Red, 3)
            End If
        Else
            If withSound Then soundPlayer.PlayClick(soundFile)
        End If
    End Sub

    ' for invoke Blip from main thread
    Private Sub RecorderInfoThreadMethod(ByVal recState As Boolean)
        If recState = True Then
            picTitleBar.SetMessage2("Recording...", Color.Orange, maxRecDuration)
            soundPlayer.BlipRecordStart()
        Else
            soundPlayer.BlipRecordStop()
            picTitleBar.ResetMessage("Recording...")
            recording = False
        End If
    End Sub
    Private Sub RecordVoice()
#If Not Win32 Then
        Dim fileName As String = GetISOdateString(True) & ".wav"
        Dim fullFileName As String = storageFolder & IO.Path.DirectorySeparatorChar & fileName

        Dim position As GPS_POSITION = myPos
        Dim recordStartTime As Date = Now

        mainForm.Invoke(myRecorderInfoDelegate, New Object() {True})

        Dim waveIn As WaveIn = New WaveIn(recordingBitsPerSecond, recordingSamplesPerSecond)
        waveIn.Preload(maxRecDuration * 1000, 256 * 1024)
        waveIn.Start()
        System.Threading.Thread.Sleep(maxRecDuration * 1000)

        If waveIn.Done = False Then
            waveIn.StopRecord()
        End If

        mainForm.Invoke(myRecorderInfoDelegate, New Object() {False})

        waveIn.Save(fullFileName)
        If waveIn IsNot Nothing Then
            waveIn.Dispose()
        End If
        ' HACK for messaging the user if the recording didn't work
        Dim fileInfo As New FileInfo(fullFileName)
        If Not fileInfo.Exists Then
            MsgBox("Recording didn't work. Please check settings.", MsgBoxStyle.Exclamation)
        ElseIf fileInfo.Length <= 45 Then
            MsgBox("Recording didn't work.", MsgBoxStyle.Exclamation)
            File.Delete(fullFileName)
        Else
            mainForm.Invoke(Me.AddVoiceRecordingDelegate, New Object() {position, recordStartTime, "voice", "", fileName})
        End If
#Else
        'http://social.msdn.microsoft.com/Forums/en-US/Vsexpressvb/thread/29a40321-450f-4cf4-a850-789b792abd8e
#End If
    End Sub
    Private Sub TakePicture()
#If Not Win32 Then
        Try
            Dim position As GPS_POSITION = myPos
            Dim takePictureStartTime As Date = Now
            Dim cameraCapture As Microsoft.WindowsMobile.Forms.CameraCaptureDialog = New Microsoft.WindowsMobile.Forms.CameraCaptureDialog
            cameraCapture.Mode = Microsoft.WindowsMobile.Forms.CameraCaptureMode.Still
            cameraCapture.InitialDirectory = storageFolder & IO.Path.DirectorySeparatorChar
            cameraCapture.DefaultFileName = GetISOdateString(True) & ".jpg"

            If cameraCapture.ShowDialog() = Windows.Forms.DialogResult.OK Then
                mainForm.Invoke(Me.myAddWayPointDelegate, New Object() {position, takePictureStartTime, "picture", "", cameraCapture.FileName.Substring(cameraCapture.FileName.LastIndexOf(IO.Path.DirectorySeparatorChar) + 1)})
            End If
            cameraCapture.Dispose()
            cameraCapture = Nothing
        Catch ex As Exception
            MsgBox("Could not take picture: " & ex.Message)
        End Try
#End If
    End Sub
    Public Sub StartSimulation()
        If myGPS.IsConnected Then
            myGPS.Disconnect()
        End If
        If SetupGPSDevice("simulation") Then
            myGPS.Connect()
        End If
    End Sub
    Public Sub HandleKeyPress(ByVal e As System.Windows.Forms.KeyEventArgs)
        ' used for scrolling the map
        Select Case e.KeyCode
            Case Keys.Up
                e.Handled = True
                OsmTileMap1.Scroll(OSMTileMap.ScrollDirection.UP)
            Case Keys.Down
                e.Handled = True
                OsmTileMap1.Scroll(OSMTileMap.ScrollDirection.DOWN)
            Case Keys.Left
                e.Handled = True
                OsmTileMap1.Scroll(OSMTileMap.ScrollDirection.LEFT)
            Case Keys.Right
                e.Handled = True
                OsmTileMap1.Scroll(OSMTileMap.ScrollDirection.RIGHT)
            Case Keys.Enter
                e.Handled = True
                OsmTileMap1.ExitManualMode()
#If Not Win32 Then
            Case DirectCast(HardwareKeys.ApplicationKey1, Keys)
                e.Handled = True
                HandleHwButton(1)
            Case DirectCast(HardwareKeys.ApplicationKey2, Keys)
                e.Handled = True
                HandleHwButton(2)
            Case DirectCast(HardwareKeys.ApplicationKey3, Keys)
                e.Handled = True
                HandleHwButton(3)
            Case DirectCast(HardwareKeys.ApplicationKey4, Keys)
                e.Handled = True
                HandleHwButton(4)
            Case DirectCast(HardwareKeys.ApplicationKey5, Keys)
                e.Handled = True
                HandleHwButton(5)
            Case DirectCast(HardwareKeys.ApplicationKey6, Keys)
                e.Handled = True
                HandleHwButton(6)
#End If
            Case Else
                Dim keychar As Char
                If e.Shift = True Then
                    keychar = Char.ToUpper(Chr(e.KeyCode))
                Else
                    keychar = Char.ToLower(Chr(e.KeyCode))
                End If
                Dim keyaction As New KeyAction(keychar)
                If ButtonXML.KeyActions.Contains(keyaction) Then
                    e.Handled = True
                    keyaction = ButtonXML.KeyActions.Find(keyaction).Value
                    HandleButtonAction(keyaction.btn)
                End If
        End Select
    End Sub
    Private Delegate Sub EmptyDelegate()
    Private fixChangedDelegate As EmptyDelegate = New EmptyDelegate(AddressOf FixChanged)
    Private Sub FixChanged()
        If playSoundOnFixChange Then
            If myGPS.HasFix Then
                soundPlayer.FixAquired()
            Else
                soundPlayer.FixLost()
            End If
        End If
        UpdateButtons()
        UpdateGPSpanel()
    End Sub
    Private Sub myGPS_FixChanged() Handles myGPS.FixChanged
        mainForm.Invoke(Me.fixChangedDelegate)
    End Sub
    Private PositionChangedDelegate As EmptyDelegate = New EmptyDelegate(AddressOf PositionChanged)
    Private Sub PositionChanged()
        UpdateGPSPosition()
        If trackLoggingDisabled = False AndAlso TrackLoggingTimer.Interval = 0 Then
            LogTrackPoint()
        End If
        UpdateGPSpanel()
    End Sub
    Private Sub myGPS_PositionChanged(ByRef gpsposition As GPS_POSITION) Handles myGPS.PositionChanged
        ' HACK to avoid crash on exit
        Try
            myPos = gpsposition
            mainForm.Invoke(Me.PositionChangedDelegate)
        Catch ex As System.ObjectDisposedException
        End Try
    End Sub
    Private Sub SetAutoDownload()
#If Not Win32 Then
        If phoneState Is Nothing Then
            phoneState = New PhoneState
        End If
        If autoDownload = AutoDownloadState.WithGPRS Then
            OsmTileMap1.DownloadTiles = phoneState.HasGPRSCoverage
        ElseIf autoDownload = AutoDownloadState.WithGPRSnoRoaming Then
            OsmTileMap1.DownloadTiles = phoneState.HasGPRSCoverage And Not phoneState.IsRoaming
        End If
#End If
    End Sub
#If Not Win32 Then
    Private Sub phoneState_DataStateChanged() Handles PhoneState.DataStateChanged
        If Not autoDownload = AutoDownloadState.Manual Then
            Dim newState As Boolean = OsmTileMap1.DownloadTiles
            If autoDownload = AutoDownloadState.WithGPRSnoRoaming Then
                autoDownload = AutoDownloadState.WithGPRSnoRoaming
                newState = phoneState.HasGPRSCoverage And Not phoneState.IsRoaming
            ElseIf autoDownload = AutoDownloadState.WithGPRS Then
                newState = phoneState.HasGPRSCoverage
            End If
            If newState <> OsmTileMap1.DownloadTiles Then
                OsmTileMap1.DownloadTiles = newState
                If OsmTileMap1.DownloadTiles Then
                    picTitleBar.SetMessage2("Download is ON", Color.LightYellow, 3)
                Else
                    picTitleBar.SetMessage2("Download is OFF", Color.LightGray, 3)
                End If
            End If
        End If
    End Sub
#End If
    Public ReadOnly Property DataFolder() As String
        Get
            Return storageFolder
        End Get
    End Property
    Public ReadOnly Property IsConnected() As Boolean
        Get
            Return myGPS.IsConnected
        End Get
    End Property
    Public ReadOnly Property HasFix() As Boolean
        Get
            Dim fix As Boolean = False
            If Not myGPS Is Nothing Then
                fix = myGPS.HasFix
            End If
            Return fix
        End Get
    End Property
    Public ReadOnly Property WayPoints() As SingleLinkedList(Of GPXFile.OSMWayPoint)
        Get
            If myGPXfile Is Nothing Then
                Return New SingleLinkedList(Of GPXFile.OSMWayPoint)
            Else
                Return myGPXfile.GetWayPoints
            End If
        End Get
    End Property
    Public ReadOnly Property TrackLogging() As Boolean
        Get
            Return Not trackLoggingDisabled
        End Get
    End Property
    Public ReadOnly Property TSoundPlayer() As SoundPlayer
        Get
            Return soundPlayer
        End Get
    End Property
    Public ReadOnly Property RecordingDuration() As Integer
        Get
            Return maxRecDuration
        End Get
    End Property
    ''' <summary>
    ''' Gets the GPS device.
    ''' </summary>
    ''' <value>The GPS device.</value>
    Public ReadOnly Property GPSDevice() As GPS.GPSDevice
        Get
            Return myGPS
        End Get
    End Property
End Class
