' Copyright 2007 Rubke <interrubke@gmail.com>
' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>: own class and double buffering
' Copyright 2008 - 2009 Jozef Vince <jozef.vince@freemap.sk>: optimisations, battery indicator and messages
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Public Class PictureTitleBar
    Inherits PictureBox

    Public Structure StatusMessage
        Public Message As String
        Public TimeOut As DateTime
        Public DisplayColor As Color
    End Structure

    Private doubleBuffer As Bitmap

    ' basic properties
    Private isFixed As Boolean
    Private isConnected As Boolean
    Private satellitesInUse As Integer
    Private currentSpeed As Double
    Private currentBearing As Double
    Private trackingEnabled As Boolean

    ' extended properties
    '<summary>
    ' list of messages to display in title-bar-area
    '</summary>
    Private messageList As New List(Of StatusMessage)
    Private logo As Bitmap
    Private updateCounter As Long

#If Not Win32 Then
    Private batteryStatus As New BatteryStatus
#End If

    Private backgroundBrush As New SolidBrush(Color.SlateGray)
    Private blackBrush As New SolidBrush(Color.Black)

    Private connectedBrush As New SolidBrush(Color.LightGreen)
    Private connectedPen As New Pen(Color.Green, 1)
    Private connectedActivePen As New Pen(Color.GreenYellow, 1)
    Private connectedActiveNoTrackingPen As New Pen(Color.Orange, 1)

    Private noFixBrush As New SolidBrush(Color.DarkOrange)
    Private noFixPen As New Pen(Color.Black, 1)
    Private noFixActivePen As New Pen(Color.Orange, 1)
    Private noFixActiveNoTrackingPen As New Pen(Color.OrangeRed, 1)

    Private noGpsBrush As New SolidBrush(Color.Black)
    Private noGpsPen As New Pen(Color.DarkRed, 1)
    Private noGpsActivePen As New Pen(Color.Red, 1)

    Private barNumber As Integer = 0

    Private bars(0 To 4) As Rectangle
    Private satInUseCounts() As Integer = {2, 3, 4, 6, 8}

    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        If doubleBuffer Is Nothing Then
            Repaint()
        End If
        pe.Graphics.DrawImage(doubleBuffer, 0, 0)
    End Sub
    Protected Overrides Sub OnPaintBackground(ByVal pe As System.Windows.Forms.PaintEventArgs)
        ' needs to be empty, otherwise you see the title bar flashing
    End Sub
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        MyBase.OnResize(e)
        Repaint(True)
    End Sub

    Public Sub SetMessage2(ByVal message As String, ByVal displayColor As Color, Optional ByVal timeOut As Long = 60)
        Dim m As StatusMessage = New StatusMessage

        m.Message = message
        m.TimeOut = DateAdd(DateInterval.Second, timeOut, Now())
        m.DisplayColor = displayColor

        messageList.Add(m)

        Me.Invalidate()
        Repaint()
    End Sub

    Public Sub SetMessage(ByVal message As String)
        Dim m As StatusMessage = New StatusMessage

        m.Message = message
        m.TimeOut = DateAdd(DateInterval.Second, 60, Now())
        m.DisplayColor = Color.Orange

        messageList.Add(m)

        Me.Invalidate()
        Repaint()
    End Sub

    Public Sub ResetMessage(ByVal message As String)
        Dim i As Integer
        Dim m As StatusMessage
        For i = messageList.Count - 1 To 0 Step -1
            m = messageList.Item(i)
            If m.Message = message Then
                messageList.RemoveAt(i)

                Repaint()
                Me.Invalidate()

                Exit For
            End If
        Next i
    End Sub

    Public Sub UpdateData(ByVal fix As Boolean, ByVal con As Boolean, ByVal satellitesInUse As Integer, ByVal speed As Double, ByVal bearing As Double, ByVal tracking As Boolean)
        ' allow to message time-outs even if there is no GPS data change
        updateCounter += 1
        If updateCounter = 5 Then
            updateCounter = 0
        End If

        If con = True OrElse (updateCounter = 0) OrElse Not (Me.isFixed = fix AndAlso Me.isConnected = con AndAlso Me.satellitesInUse = satellitesInUse AndAlso Me.currentSpeed = speed AndAlso Me.currentBearing = bearing) Then
            Me.isFixed = fix
            Me.isConnected = con
            Me.satellitesInUse = satellitesInUse
            Me.currentSpeed = speed
            Me.currentBearing = bearing
            Me.trackingEnabled = tracking
            Repaint()
            Me.Invalidate()
        End If
    End Sub
    Private Sub Repaint(Optional ByVal resized As Boolean = False)
        If Me.Width <= 0 Then
            Exit Sub
        End If
        If doubleBuffer Is Nothing OrElse resized Then
            doubleBuffer = New Bitmap(Me.Width, Me.Height, Imaging.PixelFormat.Format16bppRgb565)
            RepositionBars()
        End If

        Dim g As Graphics = Graphics.FromImage(doubleBuffer)
        Dim fillActiveBrush As SolidBrush
        Dim fillBrush As SolidBrush
        Dim borderPen As Pen
        Dim borderActivePen As Pen

        g.FillRectangle(backgroundBrush, 0, 0, Me.Width, Me.Height - 1)
        If isConnected Then
            If isFixed Then
                fillActiveBrush = connectedBrush
                fillBrush = backgroundBrush
                borderPen = connectedPen
                If trackingEnabled Then
                    borderActivePen = connectedActivePen
                Else
                    borderActivePen = connectedActiveNoTrackingPen
                End If
            Else
                fillActiveBrush = noFixBrush
                fillBrush = backgroundBrush
                borderPen = noFixPen
                If trackingEnabled Then
                    borderActivePen = noFixActivePen
                Else
                    borderActivePen = noFixActiveNoTrackingPen
                End If
            End If
        Else
            fillActiveBrush = blackBrush
            fillBrush = noGpsBrush
            borderPen = noGpsPen
            borderActivePen = noGpsActivePen
        End If

        Dim i As Integer
        For i = 0 To 4
            If satellitesInUse >= satInUseCounts(i) Then
                g.FillRectangle(fillActiveBrush, bars(i))
            Else
                g.FillRectangle(fillBrush, bars(i))
            End If

            If (barNumber = i) AndAlso (isConnected = True) Then
                g.DrawRectangle(borderActivePen, bars(i))
            Else
                g.DrawRectangle(borderPen, bars(i))
            End If
        Next

        Dim displayMessage As String = ""
        Dim messageColor As Color = Color.White

        Do
            If messageList.Count = 0 Then
                ' nothing to show
                Exit Do
            End If

            Dim m As StatusMessage

            m = messageList.Item(messageList.Count - 1)
            If m.TimeOut < Now() Then
                ' message reached its timeout 
                messageList.RemoveAt(messageList.Count - 1)
            Else
                ' show it yet more
                displayMessage = m.Message
                messageColor = m.DisplayColor

                Exit Do
            End If
        Loop

#If Not Win32 Then
        If displayMessage = "" Then
            If isFixed Then
                Dim currentSpeedtText As String

                If currentSpeed < 30 Then
                    currentSpeedtText = currentSpeed.ToString("##0.0")
                Else
                    currentSpeedtText = currentSpeed.ToString("##0")
                End If
                displayMessage = "Spd: " & currentSpeedtText & " Batt: " & batteryStatus.GetBatteryStatusString()

            Else
                If isConnected Then
                    displayMessage = "NOFIX " & " Batt: " & batteryStatus.GetBatteryStatusString()
                Else
                    displayMessage = "NOGPS " & " Batt: " & batteryStatus.GetBatteryStatusString()
                End If
            End If
        End If
#Else
        If displayMessage = "" Then
            If isFixed Then
                displayMessage = "Spd: " & currentSpeed.ToString("##0")
            Else
                If isConnected Then
                    displayMessage = "NOFIX "
                Else
                    displayMessage = "NOGPS "
                End If
            End If
        End If
#End If

        If (logo Is Nothing) Then
            logo = New Bitmap(GetResourceStream("logo.gif"))
        End If
        g.DrawImage(logo, 2, (Me.Height - logo.Height) >> 1)

        Dim fnt As Font = New Font("Arial", 10, FontStyle.Bold)
        g.DrawString(displayMessage, fnt, blackBrush, 26, 3)
        g.DrawString(displayMessage, fnt, New SolidBrush(messageColor), 25, 2)
        g.DrawLine(noFixPen, 0, Me.Height - 1, Me.Width, Me.Height - 1)
        g.Dispose()
        If isConnected Then
            barNumber += 1
            If barNumber > 4 Then
                barNumber = 0
            End If
        End If
    End Sub

    Private Sub RepositionBars()
        Dim w As Integer
        Dim h As Integer
        Dim barWidth As Integer
        Dim barHeight As Integer
        Dim x As Integer
        Dim y As Integer

        w = Me.Width
        h = Me.Height - 1

        h = h - 4

        barWidth = h >> 2

        x = w - 5 * (barWidth + 2) - 3
        barHeight = CInt(h * 0.2)
        y = 2 + h - barHeight
        bars(0) = New Rectangle(x, y, barWidth, barHeight)

        x += barWidth + 2
        barHeight = CInt(h * 0.4)
        y = 2 + h - barHeight
        bars(1) = New Rectangle(x, y, barWidth, barHeight)

        x += barWidth + 2
        barHeight = CInt(h * 0.6)
        y = 2 + h - barHeight
        bars(2) = New Rectangle(x, y, barWidth, barHeight)

        x += barWidth + 2
        barHeight = CInt(h * 0.8)
        y = 2 + h - barHeight
        bars(3) = New Rectangle(x, y, barWidth, barHeight)

        x += barWidth + 2
        barHeight = h '* 100 / 100
        y = 2 + h - barHeight
        bars(4) = New Rectangle(x, y, barWidth, barHeight)
    End Sub


    Private Sub PictureTitleBar_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        doubleBuffer = Nothing
    End Sub

    Protected Overrides Sub Finalize()
        MyBase.Finalize()
    End Sub

    Protected Sub New()
        MyBase.New()

        'Dieser Aufruf ist fr den Komponenten-Designer erforderlich.
        InitializeComponent()

        updateCounter = 0
    End Sub
End Class
