' Copyright 2007 Rubke <interrubke@gmail.com>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Public Class SpeedGraph
    Public MaxSpeed As Integer = 25
    Public MinSpeed As Integer = 0
    Public GridInterval As Integer = 5
    Private gridMinSpeed As Integer = 0
    Private gridMaxSpeed As Integer = 25
    Private bmpOffscreen As Bitmap
    Private bmpGrid As Bitmap
    Private values() As Integer
    Private storeIndex As Integer
    Private storeCount As Integer

    ' current min and max speeds
    Public CurrentMaxSpeed As Integer = 25
    Public CurrentMinSpeed As Integer = 0

    'size and direction 
    Public MaxSpeedStep As Integer = 0
    Public MinSpeedStep As Integer = 0

    Private fillBrush As SolidBrush
    Private colBlack As SolidBrush
    Private colWhite As SolidBrush

    Private borderPen As Pen
    Private linePen As Pen
    Private gridPen As Pen

    Private txtFnt As Font

    Private autoRecalcCounter As Integer = 0

    Public Function AddPoint(ByVal value As Integer, Optional ByVal force As Boolean = False) As Boolean
        values(storeIndex) = value
        storeIndex += 1
        If storeIndex >= storeCount Then
            storeIndex = 0
        End If
        autoRecalcCounter += 1
        If autoRecalcCounter Mod 10 = 0 Then RecalcMinMax()

        If value > CurrentMaxSpeed Then
            CurrentMaxSpeed = value
            MaxSpeedStep = CInt(IIf(CurrentMaxSpeed > MaxSpeed, 1, -1))
        End If

        If value < CurrentMinSpeed Then
            CurrentMinSpeed = value
            MinSpeedStep = CInt(IIf(CurrentMinSpeed > MinSpeed, 1, -1))
        End If

        If Not MinSpeed = CurrentMinSpeed Then
            MinSpeed += MinSpeedStep
        End If

        If Not MaxSpeed = CurrentMaxSpeed Then
            MaxSpeed += MaxSpeedStep
        End If

        If force = True Then UpdateSpeedGraph(True)
    End Function
    Public Sub UpdateSpeedGraph(Optional ByVal force As Boolean = False)
        If force = True Then Me.Invalidate()
    End Sub
    Protected Overrides Sub OnPaintBackground(ByVal pe As System.Windows.Forms.PaintEventArgs)
        'MyBase.OnPaintBackground(pe)
        ' needed!!!  otherwise the speedgraph might flash
    End Sub
    Public Sub RecalcMinMax()
        Dim w As Integer = storeCount
        CurrentMaxSpeed = values(0)
        CurrentMinSpeed = values(0)
        Dim i As Integer
        For i = 1 To w - 1
            If values(i) > CurrentMaxSpeed Then CurrentMaxSpeed = values(i)
            If values(i) < CurrentMinSpeed Then CurrentMinSpeed = values(i)
        Next

        If CurrentMaxSpeed < 25 Then
            CurrentMaxSpeed = 25
        End If

        If CurrentMaxSpeed - CurrentMinSpeed < 15 Then
            CurrentMinSpeed = CurrentMaxSpeed - 15
        End If
        CurrentMinSpeed = CInt(CurrentMinSpeed \ 5) * 5 ' round down
        CurrentMaxSpeed = CInt((CurrentMaxSpeed + 5) / 5) * 5 ' round up

        Dim GraphScale As Integer = CurrentMaxSpeed - CurrentMinSpeed

        'grid
        If GraphScale < 25 Then
            GridInterval = 5
            gridMinSpeed = CInt(CurrentMinSpeed \ 5) * 5
            gridMaxSpeed = CInt(CurrentMaxSpeed \ 5) * 5
        ElseIf GraphScale < 55 Then
            GridInterval = 10
            gridMinSpeed = CInt(CurrentMinSpeed \ 10) * 10
            gridMaxSpeed = CInt(CurrentMaxSpeed \ 10) * 10
        Else
            GridInterval = 20
            gridMinSpeed = CInt(CurrentMinSpeed \ 20) * 20
            gridMaxSpeed = CInt(CurrentMaxSpeed \ 20) * 20
        End If

        MaxSpeedStep = CInt(IIf(CurrentMaxSpeed > MaxSpeed, 1, -1))
        MinSpeedStep = CInt(IIf(CurrentMinSpeed > MinSpeed, 1, -1))
    End Sub
    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        Dim gxOff As Graphics '      //Offscreen graphics
        Dim rc As Rectangle = Me.ClientRectangle
        Dim w As Integer = storeCount
        Dim h As Integer = rc.Height - 10
        Dim i As Integer, x As Integer, y As Integer
        Dim spd As Integer
        Dim points() As Point

        If (bmpOffscreen Is Nothing) Then '//Bitmap for doublebuffering
            bmpOffscreen = New Bitmap(ClientSize.Width, ClientSize.Height, Imaging.PixelFormat.Format16bppRgb565)
        End If

        If (bmpGrid Is Nothing) Then
            bmpGrid = New Bitmap(w, 1)
            For x = 0 To w - 3 Step 2
                bmpGrid.SetPixel(x, 0, Color.Black)
                bmpGrid.SetPixel(x + 1, 0, Color.SlateGray)
            Next
        End If

        ReDim points(w - 1)

        gxOff = Graphics.FromImage(bmpOffscreen)
        gxOff.Clear(Me.BackColor)

        Dim scaledivisor As Double = (MaxSpeed - MinSpeed) / h

        ' line
        x = w - 1
        If storeIndex <> 0 Then
            For i = (storeIndex - 1) To 0 Step -1
                y = rc.Height - CInt((values(i) - MinSpeed) / scaledivisor)
                points(x) = New Point(x, y)
                x -= 1
            Next
        End If
        For i = (w - 1) To storeIndex Step -1
            y = rc.Height - CInt((values(i) - MinSpeed) / scaledivisor)
            points(x) = New Point(x, y)
            x -= 1
        Next

        Dim fillpoints(w + 1) As Point ' add 2 points
        points.CopyTo(fillpoints, 1)
        fillpoints(w + 1).X = w : fillpoints(w + 1).Y = h + 9
        fillpoints(0).X = 0 : fillpoints(0).Y = h + 9
        gxOff.FillPolygon(fillBrush, fillpoints)
        gxOff.DrawLines(linePen, points)

        If GridInterval > 0 AndAlso (MaxSpeed - MinSpeed) <> 0 Then

            ' transparent grid lines over graph area
            Dim imageAttr As Imaging.ImageAttributes = New Imaging.ImageAttributes()
            imageAttr.SetColorKey(Color.SlateGray, Color.SlateGray)
            Dim imgRect As Rectangle

            For spd = gridMinSpeed To gridMaxSpeed Step GridInterval
                y = rc.Height - CInt((spd - MinSpeed) / scaledivisor)
                imgRect = New Rectangle(0, y, bmpGrid.Width, bmpGrid.Height)
                gxOff.DrawImage(bmpGrid, imgRect, 0, 0, bmpGrid.Width, bmpGrid.Height, GraphicsUnit.Pixel, imageAttr)
            Next
        End If

        Dim spdText As String
        Dim strY As Integer
        Dim textHeight As Integer = gxOff.MeasureString("000", txtFnt).ToSize.Height

        If GridInterval > 0 AndAlso (MaxSpeed - MinSpeed) <> 0 Then
            For spd = gridMinSpeed To gridMaxSpeed Step GridInterval
                y = rc.Height - CInt((spd - MinSpeed) / scaledivisor)
                spdText = spd.ToString
                If Not spd = MinSpeed Then
                    strY = y - (textHeight >> 1)
                Else
                    strY = y - textHeight
                End If
                gxOff.DrawString(spdText, txtFnt, colBlack, 4, strY)
                gxOff.DrawString(spdText, txtFnt, colBlack, 6, strY)
                gxOff.DrawString(spdText, txtFnt, colBlack, 5, strY - 1)
                gxOff.DrawString(spdText, txtFnt, colBlack, 5, strY + 1)
                gxOff.DrawString(spdText, txtFnt, colWhite, 5, strY)
            Next
        End If

        ' border
        rc.Width -= 1
        rc.Height -= 1
        gxOff.DrawRectangle(borderPen, rc)


        'Draw from the memory bitmap
        pe.Graphics.DrawImage(bmpOffscreen, 0, 0)
        MyBase.OnPaint(pe)
    End Sub
    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        fillBrush = New SolidBrush(Color.LightGray)
        colBlack = New SolidBrush(Color.Black)
        colWhite = New SolidBrush(Color.White)

        borderPen = New Pen(Color.Black)
        linePen = New Pen(Color.White)
        gridPen = New Pen(Color.SteelBlue)

        txtFnt = New Font("Arial", 7, FontStyle.Bold)
    End Sub
    Private Sub Graph_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        bmpOffscreen = Nothing
        bmpGrid = Nothing
        ReDim Preserve values(Me.Width - 3) ' 2 border pixels, and 1 for the 0 start
        storeIndex = 0
        storeCount = values.GetLength(0)
    End Sub
End Class
