' Copyright 2008 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.Xml
Imports System.Xml.XPath
Imports System.IO
Public Class XMLConfig
    Private fileName As String
    Private xmlDoc As New XmlDocument
    Public Shared myCultureInfo As New System.Globalization.CultureInfo("en-US")
    Public Sub New(ByVal filename As String)
        Me.fileName = filename
        Try
            xmlDoc.Load(filename)
        Catch e As Exception
        End Try
        Dim xmlnode As XmlNode = xmlDoc.SelectSingleNode("//configuration")
        If xmlnode Is Nothing Then
            xmlnode = xmlDoc.AppendChild(xmlDoc.CreateElement("configuration"))
        End If
        If xmlDoc.SelectSingleNode("//configuration/appSettings") Is Nothing Then
            xmlnode.AppendChild(xmlDoc.CreateElement("appSettings"))
        End If
    End Sub
    Public Sub SetSetting(ByVal parametername As String, ByVal value As String)
        Dim node As XmlNode = xmlDoc.SelectSingleNode("configuration/appSettings/add[@key='" + parametername + "']")
        If node Is Nothing Then
            node = xmlDoc.SelectSingleNode("//configuration/appSettings")
            node = node.AppendChild(xmlDoc.CreateElement("add"))
            Dim xmlattribute As XmlAttribute = xmlDoc.CreateAttribute("key")
            xmlattribute.Value = parametername
            node.Attributes.Append(xmlattribute)
            node.Attributes.Append(xmlDoc.CreateAttribute("value"))
        End If
        node.Attributes.GetNamedItem("value").Value = value
        xmlDoc.Save(fileName)
    End Sub
    Public Sub SetSetting(ByVal parametername As String, ByVal value As Integer)
        SetSetting(parametername, value.ToString)
    End Sub
    Public Sub SetSetting(ByVal parametername As String, ByVal value As Double)
        SetSetting(parametername, value.ToString(myCultureInfo))
    End Sub
    Public Sub SetSetting(ByVal parametername As String, ByVal value As Boolean)
        If value = True Then
            SetSetting(parametername, "1")
        Else
            SetSetting(parametername, "0")
        End If
    End Sub
    Public Sub SetSetting(ByVal parametername As String, ByVal value As Color)
        SetSetting(parametername, (value.R * 256 * 256 + value.G * 256 + value.B).ToString("X6"))
    End Sub
    Public Function GetSetting(ByVal parametername As String, Optional ByVal defaultValue As String = "") As String
        Dim node As XmlNode = xmlDoc.SelectSingleNode("configuration/appSettings/add[@key='" + parametername + "']")
        If node Is Nothing OrElse node.Attributes("value") Is Nothing Then
            Return defaultValue
        Else
            Return node.Attributes("value").Value
        End If
    End Function
    Public Function GetIntegerSetting(ByVal parametername As String, Optional ByVal defaultValue As Integer = 0) As Integer
        Try
            Return Integer.Parse(GetSetting(parametername, CStr(defaultValue)))
        Catch e As Exception
            Return defaultValue
        End Try
    End Function
    Public Function GetDoubleSetting(ByVal parametername As String, Optional ByVal defaultValue As Double = 0) As Double
        Try
            Return Double.Parse(GetSetting(parametername, CStr(defaultValue)).Replace(","c, "."c), myCultureInfo)
        Catch e As Exception
            Return defaultValue
        End Try
    End Function
    Public Function GetBooleanSetting(ByVal parametername As String, Optional ByVal defaultValue As Boolean = False) As Boolean
        Dim result As String
        If defaultValue = True Then
            result = GetSetting(parametername, "1")
        Else
            result = GetSetting(parametername, "0")
        End If
        Return result = "1"
    End Function
    Public Function GetColorSetting(ByVal parametername As String, ByVal defaultValue As Color) As Color
        Try
            Return Color.FromArgb(Integer.Parse(Mid(GetSetting(parametername), 1, 2), Globalization.NumberStyles.HexNumber), Integer.Parse(Mid(GetSetting(parametername), 3, 2), Globalization.NumberStyles.HexNumber), Integer.Parse(Mid(GetSetting(parametername), 5, 2), Globalization.NumberStyles.HexNumber))
        Catch e As Exception
            Return defaultValue
        End Try
    End Function
    Public Sub DeleteSetting(ByVal parametername As String)
        Dim node As XmlNode = xmlDoc.SelectSingleNode("configuration/appSettings/add[@key='" + parametername + "']")
        If node IsNot Nothing Then
            xmlDoc.SelectSingleNode("//configuration/appSettings").RemoveChild(node)
            xmlDoc.Save(fileName)
        End If
    End Sub
End Class

