' Copyright 2007 Rubke <interrubke@gmail.com>
' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Public Class frmMain
    Private WithEvents logic As OSMtrackerLogic
    ' HACK NOTICE!!! ensure to update this ENUM when tab order or content is changed
    Private Enum TabNames As Integer
        Waypoints = 0
        Data = 1
        GPS = 2
        Map = 3
        Debug = 4
    End Enum
    Public Sub New()
        frmSplashScreen.Show()
        Application.DoEvents()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        logic = New OSMtrackerLogic(Me, OsmTileMap1, picTitleBar)
        logic.AfterCreationSetup()

        Application.DoEvents()
        frmSplashScreen.Close()
        Me.ControlBox = False
        Me.FormBorderStyle = Windows.Forms.FormBorderStyle.FixedDialog
    End Sub
    Private Sub logic_FontSizeChanged(ByVal newSize As Integer) Handles logic.FontSizeChanged
        Dim fnt As New Font(TabControl1.Font.Name, newSize, TabControl1.Font.Style)
        TabControl1.Font = fnt
        btnZoomIn.Font = fnt
        btnZoomOut.Font = fnt
        btnExitManualMode.Font = fnt
    End Sub
    Private Sub UpdateGPSpanel(ByVal myPos As GPS_POSITION) Handles logic.UpdateGPSInformation
        lblInfo1.Text = "Pos: " & myPos.Latitude.ToString("0.000000") & "  " & myPos.Longitude.ToString("0.000000")
        lblInfo2.Text = "Alt: " & myPos.AltitudeWRTSeaLevel & " Spd: " & logic.GPSDevice.Speed.ToString("0.0") & " Sat: " & myPos.SatelliteCount & " " & logic.GPSDevice.FixTypeString
        lblInfo3.Text = "HVP dop: " & myPos.HorizontalDilutionOfPrecision.ToString("0.0") & "  " & myPos.VerticalDilutionOfPrecision.ToString("0.0") & "  " & myPos.PositionDilutionOfPrecision.ToString("0.0")
        If logic.HasFix AndAlso myPos.UTCTime.Year > 0 Then
            lblInfo4.Text = "Time: " & myPos.UTCTime.ToLocalTime.ToShortDateString & " " & myPos.UTCTime.ToLocalTime.ToLongTimeString
        Else
            lblInfo4.Text = "Time not available"
        End If
    End Sub
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub
    Private Sub mnuConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuConnect.Click
        logic.Connect()
    End Sub
    Private Sub mnuDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDisconnect.Click
        logic.Disconnect()
        mnuGPSsim.Enabled = True
    End Sub
    Private Sub logic_UpdateGUIByViewTimer(ByVal position As GPS_POSITION) Handles logic.UpdateGUIByViewTimer
        If logic.IsConnected AndAlso logic.HasFix Then
            SpeedGraph.AddPoint(CInt(logic.GPSDevice.Speed), TabControl1.SelectedIndex = TabNames.GPS)
        End If
        picTitleBar.UpdateData(logic.HasFix, logic.IsConnected, position.SatelliteCount, logic.GPSDevice.Speed, logic.GPSDevice.Bearing, logic.TrackLogging)
    End Sub
    Private Sub logic_UpdateGPSPosOnMap(ByVal position As GPS_POSITION) Handles logic.UpdateGPSPosOnMap
        OsmTileMap1.SetGPSPosition(position, TabControl1.SelectedIndex = TabNames.Map)
    End Sub
    Private Sub mnuSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuSettings.Click
        logic.ShowSettings("")
    End Sub
    Private Sub logic_TrackPointAdded(ByVal numOfPoints As Integer) Handles logic.TrackPointAdded
        lblTrackPoints.Text = "#trackpoints: " & numOfPoints
    End Sub
    Private Sub FillPosListView() Handles logic.UpdateDataList
        Dim oWpt As GPXFile.OSMWayPoint
        Dim oitem As ListViewItem
        Dim i As Long = 1

        With PoiListView
            .SuspendLayout()
            .Items.Clear()
            For Each oWpt In logic.WayPoints
                oitem = New ListViewItem
                oitem.Text = i.ToString
                oitem.SubItems.Add(oWpt.Name) ' name
                oitem.SubItems.Add(oWpt.GetLatitudeString) ' lat
                oitem.SubItems.Add(oWpt.GetLongitudeString) ' lon
                oitem.SubItems.Add(oWpt.Time)
                oitem.SubItems.Add(oWpt.LinkHref)
                .Items.Insert(0, oitem)
                i += 1
            Next
            .ResumeLayout()
            .Refresh()
        End With
    End Sub
    Private Sub mnuPOIdelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPOIdelete.Click
        If PoiListView.SelectedIndices.Count > 0 AndAlso MsgBox("Delete waypoint(s), Are you sure?", MsgBoxStyle.OkCancel) = MsgBoxResult.Ok Then
            Dim index As Integer
            For Each index In PoiListView.SelectedIndices
                logic.DeleteWaypoint(PoiListView.Items.Count - index - 1, logic.DataFolder & IO.Path.DirectorySeparatorChar & PoiListView.Items(index).SubItems(5).Text)
            Next
            FillPosListView()
        End If
    End Sub
    Private Sub mnuPOIplayAudio_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuPOIplayAudio.Click
        PlayListAudio()
    End Sub
    Private Sub UpdateButtons(ByVal myPos As GPS_POSITION) Handles logic.UpdateGUIButtons
        mnuConnect.Enabled = Not logic.IsConnected
        mnuDisconnect.Enabled = Not mnuConnect.Enabled
        mnuGPSsyncTime.Enabled = logic.HasFix

        ButtonPanel1.SetupPage(logic.HasFix)
    End Sub
    Private Sub ContextMenu1_Popup(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ContextMenuGridView.Popup
        Dim strUrl As String
        Select Case PoiListView.SelectedIndices.Count
            Case 1 ' only one entry selected
                strUrl = PoiListView.Items(PoiListView.SelectedIndices.Item(0)).SubItems(5).Text
                mnuPOIdelete.Enabled = True
                mnuPOIplayAudio.Enabled = (strUrl.Length > 0) And strUrl.EndsWith(".wav")
            Case Else
                mnuPOIplayAudio.Enabled = False
                mnuPOIdelete.Enabled = False
        End Select
    End Sub
    Private Sub mnuGPSsyncTime_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuGPSsyncTime.Click
        logic.SyncTime()
    End Sub
    Private Sub PoiListView_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles PoiListView.KeyPress
        PlayListAudio()
    End Sub
    Private Sub PoiListView_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PoiListView.SelectedIndexChanged
        PlayListAudio()
    End Sub
    Private Sub PlayListAudio()
        If PoiListView.SelectedIndices.Count = 1 Then
            Dim strUrl As String
            strUrl = PoiListView.Items(PoiListView.SelectedIndices.Item(0)).SubItems(5).Text
            If strUrl.Length > 0 And strUrl.EndsWith(".wav") Then
                Dim index As Integer = PoiListView.SelectedIndices.Item(0)
                Dim strFile As String = PoiListView.Items(index).SubItems(5).Text
                Dim strFileName As String = logic.DataFolder & IO.Path.DirectorySeparatorChar & strFile
                picTitleBar.SetMessage2("Playing...", Color.LightGreen, logic.RecordingDuration)
                Application.DoEvents()
                logic.TSoundPlayer.PlayClip(strFileName)
            End If
        End If
    End Sub
    Private Sub ButtonPanel1_ButtonHitEx(ByRef button As Button) Handles ButtonPanel1.ButtonHitEx
        logic.HandleButtonAction(button)
    End Sub
    Private Sub mnuGPSsim_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuGPSsim.Click
        logic.StartSimulation()
    End Sub
    Private Sub mnuAbout2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuAbout.Click
        frmAbout.ShowDialog()
    End Sub
    Private Sub frmMain_Down(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        logic.HandleKeyPress(e)
    End Sub
    Private Sub frmMain_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        logic.ShuttingDown()
    End Sub
    Private Sub picTitleBar_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picTitleBar.MouseUp
        ContextMenuFile.Show(Me, New Point(0, picTitleBar.Height - 1))
    End Sub
    Private Sub btnZoomIn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomIn.Click
        btnZoomIn.Enabled = False
        OsmTileMap1.ZoomIn()
    End Sub
    Private Sub btnZoomOut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomOut.Click
        btnZoomOut.Enabled = False
        OsmTileMap1.ZoomOut()
    End Sub
    Private Sub TabControl1_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles TabControl1.SelectedIndexChanged
        Select Case TabControl1.SelectedIndex
            Case TabNames.Map
                TabPage4.Focus()
                'UpdateGPSPosition()
            Case TabNames.GPS
                SpeedGraph.UpdateSpeedGraph(True)
        End Select
    End Sub
    Private Sub mnuOpenGPX_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles mnuOpenGPX.Click
        Dim dialog As frmFileFolderSelectDialog = New frmFileFolderSelectDialog("*." & GPXFile.GPX_EXTENSION)
        With dialog
            .SelectedElement = logic.DataFolder
            If .ShowDialog = Windows.Forms.DialogResult.OK Then
                logic.OpenRouteGPXFile(.SelectedElement)
            End If
        End With
    End Sub
    Private Sub mnuNewGPX_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuNewGPX.Click
        logic.NewGPXFile()
    End Sub
    Private Sub btnExitManualMode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExitManualMode.Click
        OsmTileMap1.ExitManualMode()
    End Sub
    Private Sub logic_UpdateSimulation(ByVal enabled As Boolean) Handles logic.UpdateSimulation
        mnuGPSsim.Enabled = enabled
    End Sub
    Private Sub OsmTileMap1_Zoomed() Handles OsmTileMap1.Zoomed
        btnZoomIn.Enabled = OsmTileMap1.IsZoomInAble
        btnZoomOut.Enabled = OsmTileMap1.IsZoomOutAble
    End Sub
End Class
