' Copyright 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

Imports System.IO
Imports System.Collections.Generic

''' <summary>
''' Class for handling a GPX file containing a route to display.
''' </summary>
Public Class RouteGPXFile
    Inherits GPXFile

    Protected routeTrackPoints As SingleLinkedList(Of OSMTrackPoint) = New SingleLinkedList(Of OSMTrackPoint)
    Protected tracklogGPXFile As GPXFile = Nothing
    Public Sub New(ByVal routeFilename As String, ByVal storeDoP As Boolean, ByVal tracklogGPXFile As GPXFile)
        ' HACK to not double wrap a LoggingGPXFile
        If tracklogGPXFile.GetType.Equals(GetType(RouteGPXFile)) Then
            Me.tracklogGPXFile = CType(tracklogGPXFile, RouteGPXFile).GetWrappedGpxFile
        Else
            Me.tracklogGPXFile = tracklogGPXFile
        End If
        If IO.File.Exists(routeFilename) Then
            System.Windows.Forms.Cursor.Current = Cursors.WaitCursor
            Try
                Dim gpxFile As Xml.XmlDocument = New Xml.XmlDocument
                gpxFile.Load(routeFilename)
                Dim nsmgr As New Xml.XmlNamespaceManager(gpxFile.NameTable)
                nsmgr.AddNamespace("gpx", "http://www.topografix.com/GPX/1/1")
                nsmgr.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance")
                nsmgr.AddNamespace("xsi:schemaLocation", "http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd")
                Dim nodes As Xml.XmlNodeList = gpxFile.SelectNodes("//gpx:trkpt", nsmgr)
                System.Windows.Forms.Cursor.Current = Cursors.Default
                MsgBox("start importing nodes: " & nodes.Count)
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor
                For Each xmlobj As Xml.XmlElement In nodes
                    routeTrackPoints.Add(New OSMTrackPoint(New MAP_POSITION(Double.Parse(xmlobj.Attributes("lat").InnerText, myCultureInfo), Double.Parse(xmlobj.Attributes("lon").InnerText, myCultureInfo))))
                Next
            Catch e As Exception
                System.Windows.Forms.Cursor.Current = Cursors.Default
                MsgBox("Error while loading gpx-file:" & vbCrLf & e.Message, MsgBoxStyle.Exclamation)
            End Try
            System.Windows.Forms.Cursor.Current = Cursors.Default
        End If
    End Sub
    Public Function GetWrappedGpxFile() As GPXFile
        Return tracklogGPXFile
    End Function
    Public Overrides Function GetTrackPoints() As SingleLinkedList(Of OSMTrackPoint)
        Return routeTrackPoints
    End Function
    Public Overrides Sub AddTrack()
        tracklogGPXFile.AddTrack()
    End Sub
    Public Overrides Sub AddWayPoint(ByRef position As GPS_POSITION, ByVal time As Date, ByVal name As String, ByVal comment As String, ByVal linkhref As String)
        tracklogGPXFile.AddWayPoint(position, time, name, comment, linkhref)
    End Sub
    Public Overrides Sub AddTrackPoint(ByVal position As GPS_POSITION, ByVal time As Date)
        tracklogGPXFile.AddTrackPoint(position, time)
    End Sub
    Public Overrides Sub Save()
        tracklogGPXFile.Save()
    End Sub
    Public Overrides Function GetWayPointsCount() As Integer
        Return tracklogGPXFile.GetWayPointsCount
    End Function
    Public Overrides Function GetTrackPointsCount() As Integer
        Return tracklogGPXFile.GetTrackPointsCount
    End Function
    Public Overrides Function RemoveWayPoint(ByVal index As Integer) As Boolean
        Return tracklogGPXFile.RemoveWayPoint(index)
    End Function
    Public Overrides Function GetWayPoints() As SingleLinkedList(Of OSMWayPoint)
        Return tracklogGPXFile.GetWayPoints
    End Function
End Class
