' Copyright 2008 - 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

''' <summary>
''' Class which stores a tile (number, picture).
''' </summary>
Public Class OSMTile
    ''' <summary>
    ''' Tile X position.
    ''' </summary>
    Private tileX As Integer
    ''' <summary>
    ''' Tile Y position.
    ''' </summary>
    Private tileY As Integer
    ''' <summary>
    ''' Tile zoom-level.
    ''' </summary>
    Private tileZ As Integer = 13
    ''' <summary>
    ''' The tile image.
    ''' </summary>
    Private tile As Bitmap = Nothing

    ''' <summary>
    ''' Returns the tile image.
    ''' </summary>
    ''' <returns>The tile image. Is Nothing if not available.</returns>
    Public Function GetTileImage() As Bitmap
        Return tile
    End Function
    ''' <summary>
    ''' Sets the tile image. Nothing if not available.
    ''' </summary>
    ''' <param name="newTile">The new tile.</param>
    Public Sub SetTileImage(ByVal newTile As Bitmap)
        Dispose()
        tile = newTile
    End Sub
    Public Sub Dispose()
        If tile IsNot Nothing Then
            tile.Dispose()
            tile = Nothing
        End If
    End Sub
    ''' <summary>
    ''' Gets the tile coords.
    ''' </summary>
    ''' <param name="x">The x.</param>
    ''' <param name="y">The y.</param>
    ''' <param name="z">The z.</param>
    Public Sub GetTileCoords(ByRef x As Integer, ByRef y As Integer, ByRef z As Integer)
        x = tileX
        y = tileY
        z = tileZ
    End Sub
    ''' <summary>
    ''' Sets the tile coords.
    ''' </summary>
    ''' <param name="x">The x.</param>
    ''' <param name="y">The y.</param>
    ''' <param name="z">The z.</param>
    Public Sub SetTileCoords(ByVal x As Integer, ByVal y As Integer, ByVal z As Integer)
        tileX = x
        tileY = y
        tileZ = z
    End Sub
    ''' <summary>
    ''' Sets the tile (position and image) at once.
    ''' </summary>
    ''' <param name="x">The x.</param>
    ''' <param name="y">The y.</param>
    ''' <param name="z">The z.</param>
    ''' <param name="newTile">The new tile.</param>
    Public Sub SetTile(ByVal x As Integer, ByVal y As Integer, ByVal z As Integer, ByVal newTile As Bitmap)
        SetTileCoords(x, y, z)
        SetTileImage(newTile)
    End Sub
    ''' <summary>
    ''' Determines whether the image for the tile is available.
    ''' </summary>
    ''' <returns>
    ''' <c>true</c> if the image for the tile is available; otherwise, <c>false</c>.
    ''' </returns>
    Public Function IsAvailable() As Boolean
        Return (tile IsNot Nothing)
    End Function
    ''' <summary>
    ''' Compares the current tile coordinates with the given ones.
    ''' </summary>
    ''' <param name="x">The x.</param>
    ''' <param name="y">The y.</param>
    ''' <param name="z">The z.</param>
    ''' <returns>
    ''' <c>true</c> if the tile is the "same" one; otherwise, <c>false</c>.
    ''' </returns>
    Public Function CompareCoordinates(ByVal x As Integer, ByVal y As Integer, ByVal z As Integer) As Boolean
        'Return tileZ = z AndAlso (tileX = PositiveMod(x, PowerOf2(z)) AndAlso tileY = PositiveMod(y, PowerOf2(z)))
        Return (tileX = x AndAlso tileY = y AndAlso tileZ = z)
    End Function
End Class
