' Copyright 2009 Sven Strickroth <email@cs-ware.de>
'
' This file is part of OSMtracker.
'
' OSMtracker is free software: you can redistribute it and/or modify
' it under the terms of the GNU General Public License version 3 as
' published by the Free Software Foundation.
'
' OSMtracker is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with OSMtracker. If not, see <http://www.gnu.org/licenses/>.

Option Explicit On
Option Strict On

''' <summary>
''' Enumerates the elements of a SingleLinkedList.
''' </summary>
''' <typeparam name="X"></typeparam>
Public Class SingleListEnumerator(Of X)
    Implements IEnumerator(Of X)

    ' TODO: invalidate after changes on list
    Private listHead As SingleLinkedListNode(Of X)
    Private currentNode As SingleLinkedListNode(Of X)
    Private atEnd As Boolean = False

    ''' <summary>
    ''' Initializes a new instance of the <see cref="SingleListEnumerator(Of X)" /> class.
    ''' </summary>
    ''' <param name="listHead">The list head.</param>
    Friend Sub New(ByVal listHead As SingleLinkedListNode(Of X))
        Me.listHead = listHead
        currentNode = Nothing
    End Sub
    ''' <summary>
    ''' Gets the element at the current position of the enumerator.
    ''' </summary>
    ''' <value></value>
    ''' <returns>The element in the SingleLinkedList at the current position of the enumerator.</returns>
    Public ReadOnly Property Current() As X Implements System.Collections.Generic.IEnumerator(Of X).Current
        Get
            If currentNode IsNot Nothing Then
                Return currentNode.value
            Else
                Throw New InvalidOperationException()
            End If
        End Get
    End Property
    ''' <summary>
    ''' Gets the element at the current position of the enumerator.
    ''' </summary>
    ''' <value>The element in the SingleLinkedList at the current position of the enumerator.</value>
    Public ReadOnly Property Current1() As Object Implements System.Collections.IEnumerator.Current
        Get
            If currentNode IsNot Nothing Then
                Return currentNode.value
            Else
                Throw New InvalidOperationException()
            End If
        End Get
    End Property
    ''' <summary>
    ''' Advances the enumerator to the next element of the SingleLinkedList.
    ''' </summary>
    ''' <returns>
    ''' true if the enumerator was successfully advanced to the next element; false if the enumerator has passed the end of the collection.
    ''' </returns>
    ''' <exception cref="T:System.InvalidOperationException">The collection was modified after the enumerator was created.</exception>
    Public Function MoveNext() As Boolean Implements System.Collections.IEnumerator.MoveNext
        If atEnd Then
            Return False
        ElseIf currentNode Is Nothing Then
            currentNode = listHead
        Else
            currentNode = currentNode.nextNode
        End If
        If currentNode Is Nothing Then
            atEnd = True
            Return False
        Else
            Return True
        End If
    End Function
    ''' <summary>
    ''' Sets the enumerator to its initial position, which is before the first element in the collection. This class cannot be inherited.
    ''' </summary>
    ''' <exception cref="T:System.InvalidOperationException">The collection was modified after the enumerator was created.</exception>
    Public Sub Reset() Implements System.Collections.IEnumerator.Reset
        currentNode = Nothing
        atEnd = False
    End Sub
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Not needed?!
        'GC.SuppressFinalize(Me)
    End Sub
End Class