#define version="0.10.0"

[Setup]
AppName=OSMtracker
AppVerName=OSMtracker {#version}
AppPublisherURL=http://wiki.openstreetmap.org/index.php/OSMtracker
AppSupportURL=http://wiki.openstreetmap.org/index.php/OSMtracker
AppUpdatesURL=http://wiki.openstreetmap.org/index.php/OSMtracker
AppVersion={#version}
DefaultDirName={pf}\OSMtracker
DisableDirPage=auto
AllowUNCPath=false
PrivilegesRequired=poweruser
OutputBaseFilename=OSMtracker-install-{#version}
AllowRootDirectory=false
DisableStartupPrompt=true
OutputDir=Release
UseSetupLdr=true
InternalCompressLevel=max
EnableDirDoesntExistWarning=true
UninstallDisplayName=OSMtracker {#version}
UninstallDisplayIcon={app}\logo.ico
VersionInfoVersion={#version}
VersionInfoTextVersion={#version}
ShowLanguageDialog=no
AppID={{084C38F2-07C2-4EC8-8338-950BCBA09957}
VersionInfoCopyright=OSMtracker development team
LicenseFile=..\COPYING
AllowNoIcons=true
DefaultGroupName=OSMtracker
UsePreviousGroup=true
DisableProgramGroupPage=auto
AppPublisher=OSMtracker development team
AlwaysUsePersonalGroup=false
FlatComponentsList=false
ShowTasksTreeLines=true
WizardImageFile=compiler:WizModernImage-IS.bmp
WizardSmallImageFile=compiler:WizModernSmallImage-IS.bmp
InfoBeforeFile=info.txt

[Files]
Source: OSMtracker.ini; DestDir: {app}; Components: osmtracker\pocketpc
Source: Release\OSMtracker.cab; DestDir: {app}; Components: osmtracker\pocketpc
Source: OSMtracker4SmartPhone.ini; DestDir: {app}; Components: osmtracker\smartphone
Source: Release\OSMtracker4SmartPhone.cab; DestDir: {app}; Components: osmtracker\smartphone
Source: ..\ButtonEditor\bin\Release\ButtonEditor.exe; DestDir: {app}; Components: buttoneditor
Source: ..\GPXTool\bin\Release\GPXTool.exe; DestDir: {app}; Components: gpxtool
Source: ..\COPYING; DestDir: {app}; DestName: COPYING.txt

[Run]
FileName: {code:GetCEappManager}; Parameters: {app}\osmtracker.ini; Components: osmtracker\pocketpc; Tasks: activesync
FileName: {code:GetCEappManager}; Parameters: {app}\OSMtracker4SmartPhone.ini; Components: osmtracker\smartphone; Tasks: activesync

[UninstallRun]
Filename: {code:GetCEappManager}; Tasks: activesync; Check: ActiveSyncAvailable

[Types]
Name: full; Description: Full installation
Name: pocketpc; Description: Typical installation for PocketPC/PDA
Name: smartphone; Description: Typical installation for SmartPhone
Name: extratools; Description: Typical installation for extra tools only; Check: dotnet20Available
Name: custom; Description: Custom installation; Flags: iscustom

[Components]
Name: osmtracker; Description: OSMtracker; Types: full
Name: osmtracker\pocketpc; Description: OSMtracker for PocketPC/PDA; Types: pocketpc full
Name: osmtracker\smartphone; Description: OSMtracker for SmartPhone; Types: smartphone full
Name: buttoneditor; Description: Button Editor; Types: extratools smartphone pocketpc full; Check: dotnet20Available
Name: gpxtool; Description: GPXTool; Types: extratools smartphone pocketpc full; Check: dotnet20Available

[Tasks]
Name: activesync; Description: Install OSMtracker using ActiveSync on your mobile device; Check: ActiveSyncAvailable; Components: osmtracker

[Icons]
Name: "{group}\Button Editor"; Filename: {app}\ButtonEditor.exe; WorkingDir: {app}; IconFilename: {app}\ButtonEditor.exe; IconIndex: 0; Components: buttoneditor
Name: {group}\GPXTool; Filename: {app}\GPXTool.exe; WorkingDir: {app}; IconFilename: {app}\GPXTool.exe; IconIndex: 0; Components: gpxtool
Name: {group}\COPYING; Filename: {app}\COPYING.txt; WorkingDir: {app}
Name: "{group}\OSMtracker directory"; Filename: {app}; WorkingDir: {app}; Components: osmtracker

[Code]
function GetCEappManager(Param : string) : string;
var Path: String;
begin
  Path:= '';
  RegQueryStringValue(HKEY_LOCAL_MACHINE, 'SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\CEAPPMGR.EXE', '', Path)
  result:= Path
end;

function ActiveSyncAvailable() : boolean;
begin
  result:= not (GetCEappManager('') = '');
end;

function dotnet20Available() : boolean;
var
  version: cardinal;
begin
  RegQueryDWordValue(HKEY_LOCAL_MACHINE, 'Software\Microsoft\NET Framework Setup\NDP\v2.0.50727', 'Install', version);
  result := IntToStr(version) = '1';
end;

procedure InitializeWizard();
begin
  if ActiveSyncAvailable()=false then
  begin
    WizardForm.InfoBeforeMemo.Lines.Insert(0,'If you want to automatically install OSMtracker on your mobile device you need ActiveSync 4.5 or newer on your computer. Otherwise you have to copy and execute the .CAB-file manually to your device.'+ #13 +'You can get ActiveSync here: http://www.microsoft.com/downloads/details.aspx?FamilyID=9e641c34-6f7f-404d-a04b-dc09f8141141'+#13+#10);
  end;
  if dotnet20Available()=false then
  begin
    WizardForm.InfoBeforeMemo.Lines.Add(#13+#10+'If you want to install the Button Editor you need the .NET Framework 2.0 or newer on your computer.'+ #13 +'You can download the .NET Framework here: http://msdn.microsoft.com/netframework/aa731542.aspx or direct link for x86: http://www.microsoft.com/downloads/details.aspx?familyid=79BC3B77-E02C-4AD3-AACF-A7633F706BA5');
  end;
  WizardForm.InfoBeforeMemo.SelStart:=0;
end;

